@@ -182,7 +182,8 @@ constant pool."
                    translating)))
     (loop for vop in vops
           for name = (sb-c::vop-info-name vop)
-          for loc = (sb-int:info :source-location :vop name)
+          for loc = (sb-c::vop-parse-source-location
+                     (gethash name sb-c::*backend-parsed-vops*))
           when loc
           collect (let ((source (translate-source-location loc)))
                     (setf (definition-source-description source)
@@ -313,15 +314,10 @@ If an unsupported TYPE is requested, the function will return NIL.
                       (not (eq type :condition)))
               (find-definition-source class)))))
        ((:method-combination)
-        (let ((combination-fun
-                (find-method #'sb-mop:find-method-combination
-                             nil
-                             (list (find-class 'generic-function)
-                                   (list 'eql name)
-                                   t)
-                             nil)))
-          (when combination-fun
-            (find-definition-source combination-fun))))
+        (let ((info (gethash name sb-pcl::**method-combinations**)))
+          (when info
+            (translate-source-location
+             (sb-pcl::method-combination-info-source-location info)))))
        ((:package)
         (when (symbolp name)
           (let ((package (find-package name)))
