@@ -5,8 +5,17 @@
 ;;; things.)
 (setf *print-level* 5 *print-length* 5)
 
-(progn (load "src/cold/shared.lisp")
-       (load "tools-for-build/ldso-stubs.lisp"))
+(progn
+  (load "src/cold/shared.lisp")
+  (load "tools-for-build/ldso-stubs.lisp")
+  (let ((*print-pretty* nil)
+        (*print-length* nil))
+    (dolist (thing '(("SB!XC" "*FEATURES*")
+                     ("SB-COLD" "*SHEBANG-BACKEND-SUBFEATURES*")))
+      (let* ((sym (intern (cadr thing) (car thing)))
+             (val (symbol-value sym)))
+        (when val
+          (format t "~&target ~S = ~S~%" sym  val))))))
 (in-package "SB-COLD")
 (progn
   (setf *host-obj-prefix* "obj/from-host/")
@@ -82,6 +91,7 @@
 
 ;;; propagate structure offset and other information to the C runtime
 ;;; support code.
+ (load "tools-for-build/corefile.lisp" :verbose nil)
  (host-cload-stem "src/compiler/generic/genesis" nil)
 ) ; END with-compilation-unit
 
