@@ -113,13 +113,22 @@
       (format t "~&Removed ~D doc string~:P" count)))
 
   ;; Remove source forms of compiled-to-memory lambda expressions.
-  ;; The disassembler is the major culprit for retention of these.
+  ;; The disassembler is the major culprit for retention of these,
+  ;; but there are others and I don't feel like figuring out where from.
+  ;; Globally declaiming EVAL-STORE-SOURCE-FORM 0 would work too,
+  ;; but isn't it nice to know that the logic for storing the forms
+  ;; actually works? (Yes)
   (sb-vm::map-allocated-objects
    (lambda (obj type size)
-     (declare (ignore type size))
-     (when (typep obj 'sb-c::debug-source)
-       (unless (sb-c::debug-source-namestring obj)
-         (setf (sb-c::debug-source-form obj) nil))))
+     (declare (ignore size))
+     (when (= type sb-vm:code-header-widetag)
+       (dotimes (i (sb-kernel:code-n-entries obj))
+         (let ((fun (sb-kernel:%code-entry-point obj i)))
+           (when (sb-impl::%simple-fun-lexpr fun)
+             (sb-impl::set-simple-fun-info
+              fun nil
+              (sb-kernel:%simple-fun-doc fun)
+              (sb-kernel:%simple-fun-xrefs fun)))))))
    :all)
 
   ;; Fix unknown types in globaldb
@@ -216,6 +225,16 @@
 #+sb-xref-for-internals (sb-c::repack-xref :verbose 1)
 (fmakunbound 'sb-c::repack-xref)
 
+;;; The "--with" command-line mechanism (or equivalent 'customize-' file)
+;;; is convenient for passing boolean flags, but we shouldn't reflect
+;;; that mechanism into the target for every potential compile-time option
+;;; once build is complete.
+(setq *features* (delete :sb-after-xc-core *features*))
+;;; :CONS-PROFILING is not intended to persist after build.
+;;; It was merely a way to pass in a default OPTIMIZE quality.
+;;; Use (DESCRIBE-COMPILER-POLICY) to see the policy at startup.
+(setq *features* (delete :cons-profiling *features*))
+
 (progn
   (load "src/code/shaketree")
   (sb-impl::shake-packages
@@ -245,6 +264,7 @@
         #.(find-package "SB-ASSEM")
         #.(find-package "SB-DISASSEM")
         #.(find-package "SB-IMPL")
+        #.(find-package "SB-PRETTY")
         #.(find-package "SB-KERNEL"))
        ;; Assume all and only external symbols must be retained
        (eq accessibility :external))
@@ -262,7 +282,7 @@
        ;; By default, retain any symbol with any attachments
        (or (sb-kernel:symbol-info symbol)
            (and (boundp symbol) (not (keywordp symbol)))))))
-   :verbose t :print nil)
+   :verbose nil :print nil)
   (unintern 'sb-impl::shake-packages 'sb-impl))
 
 ;;; Use historical (stupid) behavior for storing pathname namestrings
