@@ -314,6 +314,7 @@
 ;;; as FIXNUMs (unlike SAPs or tagged addresses, these will not cons).
 (defun map-objects-in-range (fun start end &optional (strict-bound t))
   (declare (type function fun))
+  (declare (dynamic-extent fun))
   (named-let iter ((start start))
   (cond
     ((< (get-lisp-obj-address start) (get-lisp-obj-address end))
@@ -383,6 +384,7 @@
 (declaim (ftype (sfunction (function &rest immobile-subspaces) null)
                 map-immobile-objects))
 (defun map-immobile-objects (function &rest subspaces) ; Perform no filtering
+  (declare (dynamic-extent function))
   (do-rest-arg ((subspace) subspaces)
     (multiple-value-bind (start end) (%space-bounds subspace)
       (map-objects-in-range function start end)))))
@@ -434,6 +436,7 @@ We could try a few things to mitigate this:
   (declare (type function fun)
            ;; KLUDGE: rest-arg and self calls do not play nice and it'll get consed
            (optimize (sb-c::recognize-self-calls 0)))
+  (declare (dynamic-extent fun))
   (when (and (= (length spaces) 1) (eq (first spaces) :all))
     (return-from map-allocated-objects
      (map-allocated-objects fun
@@ -503,7 +506,8 @@ We could try a few things to mitigate this:
        ;; prove to be an issue with concurrent systems, or with
        ;; spectacularly poor timing for closing an allocation region
        ;; in a single-threaded system.
-
+       #+sb-thread
+       (close-current-gc-region)
        (loop
           with page-size = (ash gencgc-card-bytes (- n-fixnum-tag-bits))
           ;; This magic dance gets us an unboxed aligned pointer as a
@@ -534,6 +538,26 @@ We could try a few things to mitigate this:
     (if (eq space :dynamic)
         (without-gcing (do-1-space space))
         (do-1-space space)))))
+
+#+(and sb-thread gencgc)
+;; Start with a Lisp rendition of ensure_region_closed() on the active
+;; thread's region, since users are often surprised to learn that a
+;; just-consed object can't necessarily be seen by MAP-ALLOCATED-OBJECTS.
+;; Since we're in WITHOUT-GCING, there can be no interrupts.
+;; This is probably better than calling MAP-OBJECTS-IN-RANGE on the
+;; thread's region, because then we might visit some page twice
+;; by doing that.
+;; (And seeing small consing by other threads is hopeless either way)
+(defun close-current-gc-region ()
+  (unless (eql (sap-int (current-thread-offset-sap
+                         (+ thread-alloc-region-slot 3)))
+               0)                       ; start_addr
+    (alien-funcall
+     (extern-alien "gc_close_region" (function void unsigned int))
+     (truly-the word
+                (+ (sb-thread::thread-primitive-thread sb-thread:*current-thread*)
+                   (ash thread-alloc-region-slot word-shift)))
+     1)))
 
 ;;;; MEMORY-USAGE
 
@@ -879,6 +903,7 @@ We could try a few things to mitigate this:
   (declare (type spaces space)
            (type (or index null) larger smaller type count)
            (type (or function null) test))
+  (declare (dynamic-extent test))
   (unless *ignore-after*
     (setq *ignore-after* (cons 1 2)))
   (collect ((counted 0 1+))
@@ -905,6 +930,7 @@ We could try a few things to mitigate this:
 ;;; because we know that there's a stack reference.
 (defun map-stack-references (function)
   (declare (type function function))
+  (declare (dynamic-extent function))
   (macrolet ((iter (step limit test)
                `(do ((sp (current-sp) (sap+ sp (,step n-word-bytes)))
                      (limit (sb-di::descriptor-sap ,limit))
