@@ -116,19 +116,14 @@ This is SETFable."
   ;;        The alien is allocated on the heap, and has infinite extent. The alien
   ;;        is allocated at load time, so the same piece of memory is used each time
   ;;        this form executes.
-  (/show "entering WITH-ALIEN" bindings)
-  (let (#!+c-stack-is-control-stack
-        bind-alien-stack-pointer)
+  (let (bind-alien-stack-pointer)
     (with-auxiliary-alien-types env
       (dolist (binding (reverse bindings))
-        (/show binding)
         (destructuring-bind
             (symbol type &optional opt1 (opt2 nil opt2p))
             binding
-          (/show symbol type opt1 opt2)
           (let* ((alien-type (parse-alien-type type env))
                  (datap (not (alien-fun-type-p alien-type))))
-            (/show alien-type)
             (multiple-value-bind (allocation initial-value)
                 (if opt2p
                     (values opt1 opt2)
@@ -139,7 +134,6 @@ This is SETFable."
                        (values opt1 nil))
                       (t
                        (values :local opt1))))
-              (/show allocation initial-value)
               (setf body
                     (ecase allocation
                       #+nil
@@ -155,14 +149,12 @@ This is SETFable."
                                    `((setq ,symbol ,initial-value)))
                                ,@body)))))
                       (:extern
-                       (/show0 ":EXTERN case")
                        `((symbol-macrolet
                              ((,symbol
                                 (%alien-value
                                  (foreign-symbol-sap ,initial-value ,datap) 0 ,alien-type)))
                            ,@body)))
                       (:local
-                       (/show0 ":LOCAL case")
                        (let* ((var (sb!xc:gensym "VAR"))
                               (initval (if initial-value (sb!xc:gensym "INITVAL")))
                               (info (make-local-alien-info :type alien-type))
@@ -177,34 +169,21 @@ This is SETFable."
                                     `((let ((,initval ,initial-value))
                                         ,@inner-body))
                                     inner-body)))
-                         (/show var initval info)
-                         #!+c-stack-is-control-stack
-                         (progn (setf bind-alien-stack-pointer t)
-                                `((let ((,var (make-local-alien ',info)))
-                                    ,@body-forms)))
-                         ;; FIXME: This version is less efficient then it needs to be, since
-                         ;; it could just save and restore the number-stack pointer once,
-                         ;; instead of doing multiple decrements if there are multiple bindings.
-                         #!-c-stack-is-control-stack
+                         (setf bind-alien-stack-pointer t)
                          `((let ((,var (make-local-alien ',info)))
-                             (multiple-value-prog1
-                                 (progn ,@body-forms)
-                               ;; No need for unwind protect here, since
-                               ;; allocation involves modifying NSP, and
-                               ;; NSP is saved and restored during NLX.
-                               ;; And in non-transformed case it
-                               ;; performs finalization.
-                               (dispose-local-alien ',info ,var))))))))))))
-      (/show "revised" body)
+                             ,@body-forms))))))))))
       (verify-local-auxiliaries-okay)
-      (/show0 "back from VERIFY-LOCAL-AUXILIARIES-OK, returning")
       `(symbol-macrolet ((&auxiliary-type-definitions&
                            ,(append *new-auxiliary-types*
                                     (auxiliary-type-definitions env))))
          ,@(cond
-             #!+c-stack-is-control-stack
              (bind-alien-stack-pointer
-              `((let ((sb!vm::*alien-stack-pointer* sb!vm::*alien-stack-pointer*))
+              ;; The LET IR1-translator will actually turn this into
+              ;; RESTORING-NSP on #-c-stack-is-control-stack to avoid
+              ;; expanding into non-standard special forms.
+              ;; And the LET has to look exactly like this, not LET*
+              ;; and no other bindings.
+              `((let ((sb!c:*alien-stack-pointer* sb!c:*alien-stack-pointer*))
                   ,@body)))
              (t
               body))))))
@@ -283,7 +262,7 @@ Examples:
                    (error
                     "cannot override the size of zero-dimensional arrays"))
                  (when (constantp size)
-                   (setf alien-type (copy-alien-array-type alien-type))
+                   (setf alien-type (copy-structure alien-type))
                    (setf (alien-array-type-dimensions alien-type)
                          (cons (constant-form-value size) (cdr dims)))))
                 (dims
@@ -328,11 +307,6 @@ Examples:
         (malloc-error bytes (get-errno))
         sap)))
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defvar *saved-fp-and-pcs* nil)
-  ;; Can't use DECLAIM since always-bound is a non-standard declaration
-  (sb!xc:proclaim '(sb!ext:always-bound *saved-fp-and-pcs*)))
-
 #!+c-stack-is-control-stack
 (declaim (inline invoke-with-saved-fp-and-pc))
 ;;; On :c-stack-is-control-stack platforms, this DEFUN must appear prior to the
@@ -413,8 +387,7 @@ null byte."
 (defun slot (alien slot)
   "Extract SLOT from the Alien STRUCT or UNION ALIEN. May be set with SETF."
   (declare (type alien-value alien)
-           (type symbol slot)
-           (optimize (inhibit-warnings 3)))
+           (type symbol slot))
   (let ((type (alien-value-type alien)))
     (etypecase type
       (alien-pointer-type
@@ -430,8 +403,7 @@ null byte."
 ;;; this when it can't figure out anything better.
 (defun %set-slot (alien slot value)
   (declare (type alien-value alien)
-           (type symbol slot)
-           (optimize (inhibit-warnings 3)))
+           (type symbol slot))
   (let ((type (alien-value-type alien)))
     (etypecase type
       (alien-pointer-type
@@ -446,8 +418,7 @@ null byte."
 ;;; Compute the address of the specified slot and return a pointer to it.
 (defun %slot-addr (alien slot)
   (declare (type alien-value alien)
-           (type symbol slot)
-           (optimize (inhibit-warnings 3)))
+           (type symbol slot))
   (let ((type (alien-value-type alien)))
     (etypecase type
       (alien-pointer-type
@@ -507,8 +478,7 @@ null byte."
    as the indices of the array element to access. If a pointer, one index can
    optionally be specified, giving the equivalent of C pointer arithmetic."
   (declare (type alien-value alien)
-           (type list indices)
-           (optimize (inhibit-warnings 3)))
+           (type list indices))
   (multiple-value-bind (target-type offset) (deref-guts alien indices)
     (%alien-value (alien-value-sap alien)
                   offset
@@ -516,8 +486,7 @@ null byte."
 
 (defun %set-deref (alien value &rest indices)
   (declare (type alien-value alien)
-           (type list indices)
-           (optimize (inhibit-warnings 3)))
+           (type list indices))
   (multiple-value-bind (target-type offset) (deref-guts alien indices)
     (setf (%alien-value (alien-value-sap alien)
                         offset
@@ -526,8 +495,7 @@ null byte."
 
 (defun %deref-addr (alien &rest indices)
   (declare (type alien-value alien)
-           (type list indices)
-           (optimize (inhibit-warnings 3)))
+           (type list indices))
   (multiple-value-bind (target-type offset) (deref-guts alien indices)
     (%sap-alien (sap+ (alien-value-sap alien) (/ offset sb!vm:n-byte-bits))
                 (make-alien-pointer-type :to target-type))))
@@ -535,23 +503,20 @@ null byte."
 ;;;; accessing heap alien variables
 
 (defun %heap-alien (info)
-  (declare (type heap-alien-info info)
-           (optimize (inhibit-warnings 3)))
+  (declare (type heap-alien-info info))
   (%alien-value (heap-alien-info-sap info)
                 0
                 (heap-alien-info-type info)))
 
 (defun %set-heap-alien (info value)
-  (declare (type heap-alien-info info)
-           (optimize (inhibit-warnings 3)))
+  (declare (type heap-alien-info info))
   (setf (%alien-value (heap-alien-info-sap info)
                       0
                       (heap-alien-info-type info))
         value))
 
 (defun %heap-alien-addr (info)
-  (declare (type heap-alien-info info)
-           (optimize (inhibit-warnings 3)))
+  (declare (type heap-alien-info info))
   (%sap-alien (heap-alien-info-sap info)
               (make-alien-pointer-type :to (heap-alien-info-type info))))
 
@@ -609,14 +574,6 @@ null byte."
   (unless (local-alien-info-force-to-memory-p info)
     (error "~S isn't forced to memory. Something went wrong." alien))
   alien)
-
-;; It's not mandatory that this function not exist for x86[-64],
-;; however for sanity, it should not, because no call to it can occur.
-#!-(or x86 x86-64)
-(defun dispose-local-alien (info alien)
-  (declare (ignore info))
-  (cancel-finalization alien)
-  (free-alien alien))
 
 ;;;; the CAST macro
 
@@ -628,8 +585,7 @@ null byte."
 (defun %cast (alien target-type)
   (declare (type alien-value alien)
            (type alien-type target-type)
-           (optimize (safety 2))
-           (optimize (inhibit-warnings 3)))
+           (optimize (safety 2)))
   (if (or (alien-pointer-type-p target-type)
           (alien-array-type-p target-type)
           (alien-fun-type-p target-type))
@@ -868,7 +824,8 @@ way that the argument is passed.
 information we need to manipulate callbacks after their creation. Used for
 changing the lisp-side function they point to, invalidation, etc.")
 
-(defstruct callback-info
+(defstruct (callback-info
+            (:copier nil))
   specifier
   function ; NULL if invalid
   wrapper
@@ -880,55 +837,56 @@ changing the lisp-side function they point to, invalidation, etc.")
 (defun alien-callback-info (alien)
   (cdr (assoc (alien-sap alien) *alien-callback-info* :test #'sap=)))
 
-(defvar *alien-callbacks* (make-hash-table :test #'equal)
+(define-load-time-global *alien-callbacks* (make-hash-table :test #'equal)
   "Cache of existing callback SAPs, indexed with (SPECIFER . FUNCTION). Used for
 memoization: we don't create new callbacks if one pointing to the correct
 function with the same specifier already exists.")
 
-(defvar *alien-callback-wrappers* (make-hash-table :test #'equal)
+(define-load-time-global *alien-callback-wrappers* (make-hash-table :test #'equal)
   "Cache of existing lisp wrappers, indexed with SPECIFER. Used for memoization:
 we don't create new wrappers if one for the same specifier already exists.")
 
-(defvar *alien-callback-trampolines* (make-array 32 :fill-pointer 0 :adjustable t)
+(define-load-time-global *alien-callback-trampolines*
+    (make-array 32 :fill-pointer 0 :adjustable t
+                :initial-element #'invalid-alien-callback)
   "Lisp trampoline store: assembler wrappers contain indexes to this, and
 ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
 
 (defun %alien-callback-sap (specifier result-type argument-types function wrapper
                             &optional call-type)
   (declare #!-x86 (ignore call-type))
-  (let ((key (list specifier function)))
-    (or (gethash key *alien-callbacks*)
-        (setf (gethash key *alien-callbacks*)
-              (let* ((index (fill-pointer *alien-callback-trampolines*))
-                     ;; Aside from the INDEX this is known at
-                     ;; compile-time, which could be utilized by
-                     ;; having the two-stage assembler tramp &
-                     ;; wrapper mentioned in [1] above: only the
-                     ;; per-function tramp would need assembler at
-                     ;; runtime. Possibly we could even pregenerate
-                     ;; the code and just patch the index in later.
-                     (assembler-wrapper
-                      (alien-callback-assembler-wrapper
-                       index result-type argument-types
-                       #!+x86
-                       (if (eq call-type :stdcall)
-                           (ceiling
-                            (apply #'+
-                                   (mapcar 'alien-type-word-aligned-bits
-                                           argument-types))
-                            8)
-                           0))))
-                (vector-push-extend
-                 (alien-callback-lisp-trampoline wrapper function)
-                 *alien-callback-trampolines*)
-                ;; Assembler-wrapper is static, so sap-taking is safe.
-                (let ((sap (vector-sap assembler-wrapper)))
-                  (push (cons sap (make-callback-info :specifier specifier
-                                                      :function function
-                                                      :wrapper wrapper
-                                                      :index index))
-                        *alien-callback-info*)
-                  sap))))))
+  (ensure-gethash
+   (list specifier function) *alien-callbacks*
+   (let* ((index (fill-pointer *alien-callback-trampolines*))
+          ;; Aside from the INDEX this is known at
+          ;; compile-time, which could be utilized by
+          ;; having the two-stage assembler tramp &
+          ;; wrapper mentioned in [1] above: only the
+          ;; per-function tramp would need assembler at
+          ;; runtime. Possibly we could even pregenerate
+          ;; the code and just patch the index in later.
+          (assembler-wrapper
+           (alien-callback-assembler-wrapper
+            index result-type argument-types
+            #!+x86
+            (if (eq call-type :stdcall)
+                (ceiling
+                 (apply #'+
+                        (mapcar 'alien-type-word-aligned-bits
+                                argument-types))
+                 8)
+                0))))
+     (vector-push-extend
+      (alien-callback-lisp-trampoline wrapper function)
+      *alien-callback-trampolines*)
+     ;; Assembler-wrapper is static, so sap-taking is safe.
+     (let ((sap (vector-sap assembler-wrapper)))
+       (push (cons sap (make-callback-info :specifier specifier
+                                           :function function
+                                           :wrapper wrapper
+                                           :index index))
+             *alien-callback-info*)
+       sap))))
 
 (defun alien-callback-lisp-trampoline (wrapper function)
   (declare (function wrapper) (optimize speed))
@@ -940,6 +898,10 @@ ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
          (argument-names arguments)
          (argument-specs (cddr specifier)))
     `(lambda (args-pointer result-pointer function)
+       ;; KLUDGE: the SAP shouldn't be consed but they are, don't
+       ;; bother anyone about that sad fact
+       (declare (muffle-conditions compiler-note)
+                (optimize speed))
        ;; FIXME: the saps are not gc safe
        (let ((args-sap (int-sap
                         (sb!kernel:get-lisp-obj-address args-pointer)))
@@ -1032,15 +994,12 @@ ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
         (error "Unsupported callback argument type: ~A" type))))
 
 (defun enter-alien-callback (index return arguments)
-  (funcall (aref *alien-callback-trampolines* index)
+  (funcall (truly-the function
+                      (svref (sb!kernel:%array-data *alien-callback-trampolines*)
+                             index))
            return
            arguments))
 
-;;; To ensure that callback wrapper functions continue working even
-;;; if #'ENTER-ALIEN-CALLBACK moves in memory, access to it is indirected
-;;; through the *ENTER-ALIEN-CALLBACK* static symbol. -- JES, 2006-01-01
-(defvar *enter-alien-callback* #'enter-alien-callback)
-
 ;;;; interface (not public, yet) for alien callbacks
 
 (let ()
@@ -1056,11 +1015,10 @@ one."
     `(%sap-alien
       (%alien-callback-sap ',specifier ',result-type ',argument-types
                            ,function
-                           (or (gethash ',specifier *alien-callback-wrappers*)
-                               (setf (gethash ',specifier *alien-callback-wrappers*)
-                                     (compile nil
-                                              ',(alien-callback-lisp-wrapper-lambda
-                                                 specifier result-type argument-types env))))
+                           (ensure-gethash
+                            ',specifier *alien-callback-wrappers*
+                            ,(alien-callback-lisp-wrapper-lambda
+                              specifier result-type argument-types env))
                            ,call-type)
       ',(parse-alien-type specifier env)))))
 
