@@ -40,8 +40,11 @@
     (declare (index value-index))
     (setf (%instance-layout instance) (dd-layout-or-lose dd))
     (dolist (spec slot-specs instance)
-      (destructuring-bind (raw-type . index) (cdr spec)
-              (macrolet ((make-case ()
+      (destructuring-bind (kind raw-type . index) spec
+        (if (eq kind :unbound)
+            (setf (%instance-ref instance index)
+                  (sb!sys:%primitive make-unbound-marker))
+            (macrolet ((make-case ()
                            `(ecase raw-type
                               ((t)
                                (setf (%instance-ref instance index) value))
@@ -54,7 +57,7 @@
                                  *raw-slot-data*))))
                 (let ((value (fast-&rest-nth value-index slot-values)))
                   (incf value-index)
-                  (make-case)))))))
+                  (make-case))))))))
 
 (defun %instance-layout (instance)
   (%instance-layout instance))
@@ -100,8 +103,6 @@
 
   (let* ((classoid (find-classoid (dd-name dd)))
          (layout (classoid-layout classoid)))
-    (when (eq (dd-pure dd) t)
-      (setf (layout-pure layout) t))
     ;; Make a vector of EQUALP slots comparators, indexed by (- word-index data-start).
     ;; This has to be assigned to something regardless of whether there are
     ;; raw slots just in case someone mutates a layout which had raw
@@ -262,5 +263,24 @@
 
 (defmethod print-object ((x structure-object) stream)
   (default-structure-print x stream *current-level-in-print*))
+
+;;; Used internally, but it would be nice to provide something
+;;; like this for users as well.
+(defmacro define-structure-slot-addressor (name &key structure slot)
+  (let* ((dd (find-defstruct-description structure t))
+         (slotd (or (and dd (find slot (dd-slots dd) :key #'dsd-name))
+                    (error "Slot ~S not found in ~S." slot structure)))
+         (index (dsd-index slotd)))
+    `(progn
+       (declaim (inline ,name))
+       (defun ,name (instance)
+         (declare (type ,structure instance) (optimize speed))
+         (truly-the
+          word
+          (+ (get-lisp-obj-address instance)
+             ,(+ (- sb!vm:instance-pointer-lowtag)
+                 (* (+ sb!vm:instance-slots-offset index)
+                    sb!vm:n-word-bytes))))))))
+
 
 (/show0 "target-defstruct.lisp end of file")
