@@ -16,32 +16,34 @@
 
 ;;;; :CODE-OBJECT fixups
 
-#!+immobile-space
-(defun sb!kernel::immobile-space-obj-p (obj)
-  (<= immobile-space-start (get-lisp-obj-address obj) immobile-space-end))
-
 ;;; This gets called by LOAD to resolve newly positioned objects
 ;;; with things (like code instructions) that have to refer to them.
-(defun fixup-code-object (code offset fixup kind &optional flavor)
+;;; Return :ABSOLUTE if an absolute fixup needs to be recorded in %CODE-FIXUPS,
+;;; and return :RELATIVE if a relative fixup needs to be recorded.
+;;; The code object we're fixing up is pinned whenever this is called.
+(defun fixup-code-object (code offset fixup kind flavor)
   (declare (type index offset) (ignorable flavor))
-  (without-gcing
-    (let ((sap (code-instructions code)))
-      (ecase kind
+  (let* ((sap (code-instructions code))
+         (fixup (+ (if (eq kind :absolute64)
+                       (signed-sap-ref-64 sap offset)
+                       (signed-sap-ref-32 sap offset))
+                   fixup)))
+    (ecase kind
         (:absolute64
          ;; Word at sap + offset contains a value to be replaced by
          ;; adding that value to fixup.
-         (setf (sap-ref-64 sap offset) (+ fixup (sap-ref-64 sap offset))))
+         (setf (sap-ref-64 sap offset) fixup))
         (:absolute
          ;; Word at sap + offset contains a value to be replaced by
          ;; adding that value to fixup.
-         (setf (sap-ref-32 sap offset) (+ fixup (signed-sap-ref-32 sap offset))))
+         (setf (sap-ref-32 sap offset) fixup))
         (:relative
          ;; Fixup is the actual address wanted.
          ;; Replace word with value to add to that loc to get there.
          ;; In the #!-immobile-code case, there's nothing to assert.
          ;; Relative fixups pretty much can't happen.
          #!+immobile-code
-         (unless (<= immobile-space-start (get-lisp-obj-address code) immobile-space-end)
+         (unless (immobile-space-obj-p code)
            (error "Can't compute fixup relative to movable object ~S" code))
          (setf (signed-sap-ref-32 sap offset)
                (etypecase fixup
@@ -49,33 +51,26 @@
                   ;; JMP/CALL are relative to the next instruction,
                   ;; so add 4 bytes for the size of the displacement itself.
                   (- fixup
-                     (the (unsigned-byte 64) (+ (sap-int sap) offset 4))))))))))
-  ;; An absolute fixup is stored in the code header if it
+                     (the (unsigned-byte 64) (+ (sap-int sap) offset 4)))))))))
+  ;; An absolute fixup is stored in the code header's %FIXUPS slot if it
   ;; references an immobile-space (but not static-space) object.
-  ;; This needn't be inside WITHOUT-GCING, because code fixups will point
-  ;; only to objects that don't move except during save-lisp-and-die.
-  ;; So there is no race with GC here.
   ;; Note that:
-  ;;  (1) :NAMED-CALL occurs in both :RELATIVE and :ABSOLUTE kinds.
-  ;;      We can ignore the :RELATIVE kind.
-  ;;  (2) genesis does not need the :NAMED-CALL case,
-  ;;      because COMPILE-FILE always places code in immobile space,
-  ;;      and self-build does not use step instrumenting, so there are
-  ;;      no named calls using the sequence "MOV RAX, imm32 ; CALL RAX"
-  ;;      Hence SB!FASL::DO-COLD-FIXUP does not exactly mimic this logic.
+  ;;  (1) Call fixups occur in both :RELATIVE and :ABSOLUTE kinds.
+  ;;      We can ignore the :RELATIVE kind, except for foreign call.
+  ;;  (2) :STATIC-CALL fixups point to immobile space, not static space.
   #!+immobile-space
-  (when (and (eq kind :absolute) (member flavor '(:named-call :immobile-object)))
-    (let ((fixups (%code-fixups code)))
-      ;; Sanctifying the code component will compact these into a bignum.
-      (setf (%code-fixups code) (cons offset (if (eql fixups 0) nil fixups)))))
-  nil)
-
-#!+immobile-space
-(defun sanctify-for-execution (code)
-  (let ((fixups (%code-fixups code)))
-    (when (listp fixups)
-      (setf (%code-fixups code) (sb!c::pack-code-fixup-locs fixups))))
-  nil)
+  (return-from fixup-code-object
+    (case flavor
+      ((:named-call :layout :immobile-object ; -> fixedobj subspace
+        :assembly-routine :assembly-routine* :static-call) ; -> varyobj subspace
+       (if (eq kind :absolute) :absolute))
+      (:foreign
+       ;; linkage-table calls using the "CALL rel32" format need to be saved,
+       ;; because the linkage table resides at a fixed address.
+       ;; Space defragmentation can handle the fixup automatically,
+       ;; but core relocation can't - it can't find all the call sites.
+       (if (eq kind :relative) :relative))))
+  nil) ; non-immobile-space builds never record code fixups
 
 #!+(or darwin linux win32)
 (define-alien-routine ("os_context_float_register_addr" context-float-register-addr)
@@ -155,29 +150,20 @@
 ;;; arguments from the instruction stream.
 (defun internal-error-args (context)
   (declare (type (alien (* os-context-t)) context))
-  (/show0 "entering INTERNAL-ERROR-ARGS, CONTEXT=..")
-  (/hexstr context)
   (let* ((pc (context-pc context))
          (trap-number (sap-ref-8 pc 0)))
     (declare (type system-area-pointer pc))
-    (/show0 "got PC")
-    ;; using INT3 the pc is .. INT3 <here> code length bytes...
     (if (= trap-number invalid-arg-count-trap)
         (values #.(error-number-or-lose 'invalid-arg-count-error)
                 '(#.arg-count-sc))
-        (let ((error-number (sap-ref-8 pc 1)))
-          (values error-number
-                  (sb!kernel::decode-internal-error-args (sap+ pc 2) error-number)
-                  trap-number)))))
+        (sb!kernel::decode-internal-error-args (sap+ pc 1) trap-number))))
 
 
-;;; the current alien stack pointer; saved/restored for non-local exits
-(defvar *alien-stack-pointer*)
-
 #!+immobile-code
 (progn
 (defun fun-immobilize (fun)
-  (let ((code (allocate-code-object t 0 16)))
+  (let ((code (truly-the (values code-component &optional)
+                         (sb!vm::alloc-immobile-trampoline))))
     (setf (%code-debug-info code) fun)
     (let ((sap (code-instructions code))
           (ea (+ (logandc2 (get-lisp-obj-address code) lowtag-mask)
@@ -190,7 +176,7 @@
       ;; We could use the #xA1 opcode to save a byte, but that would
       ;; be another headache do deal with when relocating this code.
       ;; There's precedent for this style of hand-assembly,
-      ;; in arch_write_linkage_table_jmp() and arch_do_displaced_inst().
+      ;; in arch_write_linkage_table_entry() and arch_do_displaced_inst().
       (setf (sap-ref-32 sap 0) #x058B48 ; REX MOV [RIP-n]
             (signed-sap-ref-32 sap 3) (- ea (+ (sap-int sap) 7))) ; disp
       (let ((i (if (/= (fun-subtype fun) funcallable-instance-widetag)
@@ -206,27 +192,28 @@
 ;;; Return T if FUN can't be called without loading RAX with its descriptor.
 ;;; This is true of any funcallable instance which is not a GF, and closures.
 (defun fun-requires-simplifying-trampoline-p (fun)
-  (cond ((funcallable-instance-p fun)
+  (cond ((not (immobile-space-obj-p fun)) t) ; always
+        ((funcallable-instance-p fun)
          ;; A funcallable-instance with no raw slots has no machine
          ;; code within it, and thus requires an external trampoline.
-         (zerop (layout-bitmap (%funcallable-instance-layout fun))))
+         (eql (layout-bitmap (%funcallable-instance-layout fun))
+              sb!kernel::+layout-all-tagged+))
         (t
          (closurep fun))))
 
-(defun %set-fin-trampoline (fin)
-  (let ((sap (int-sap (- (get-lisp-obj-address fin) fun-pointer-lowtag)))
-        (insts-offs (ash (1+ funcallable-instance-info-offset) word-shift)))
-    (setf (sap-ref-word sap insts-offs) #xFFFFFFE9058B48 ; MOV RAX,[RIP-23]
-          (sap-ref-32 sap (+ insts-offs 7)) #x00FD60FF)) ; JMP [RAX-3]
-  fin)
+(defmacro !set-fin-trampoline (fin)
+  `(let ((sap (int-sap (get-lisp-obj-address ,fin)))
+         (insts-offs (- (ash (1+ funcallable-instance-info-offset) word-shift)
+                        fun-pointer-lowtag)))
+     (setf (sap-ref-word sap insts-offs) #xFFFFFFE9058B48 ; MOV RAX,[RIP-23]
+           (sap-ref-32 sap (+ insts-offs 7)) #x00FD60FF))) ; JMP [RAX-3]
 
 (defun %set-fdefn-fun (fdefn fun)
   (declare (type fdefn fdefn) (type function fun)
            (values function))
   (unless (eql (sb!vm::fdefn-has-static-callers fdefn) 0)
     (sb!vm::remove-static-links fdefn))
-  (let ((trampoline (when (or (>= (get-lisp-obj-address fun) (ash 1 32))
-                              (fun-requires-simplifying-trampoline-p fun))
+  (let ((trampoline (when (fun-requires-simplifying-trampoline-p fun)
                       (fun-immobilize fun)))) ; a newly made CODE object
     (with-pinned-objects (fdefn trampoline fun)
       (binding* (((fun-entry-addr nop-byte)
@@ -257,9 +244,13 @@
 ;;; Find an immobile FDEFN or FUNCTION given an interior pointer to it.
 #!+immobile-space
 (defun find-called-object (address)
-  (when (<= immobile-space-start address immobile-space-end)
-    (let ((obj (alien-funcall (extern-alien "search_immobile_space" (function long long))
-                              address)))
+  ;; The ADDRESS [sic] is actually any immediate operand to MOV,
+  ;; which in general decodes as a *signed* integer. So ignore negative values.
+  (let ((obj (if (typep address 'sb!ext:word)
+                 (alien-funcall (extern-alien "search_all_gc_spaces"
+                                              (function unsigned unsigned))
+                                address)
+                 0)))
       (unless (eql obj 0)
         (case (sap-ref-8 (int-sap obj) 0)
          (#.fdefn-widetag
@@ -274,7 +265,7 @@
                           (ash simple-fun-code-offset word-shift)
                           (- fun-pointer-lowtag))
                        address)
-                    (return f)))))))))))
+                    (return f))))))))))
 
 ;;; Compute the PC that FDEFN will jump to when called.
 #!+immobile-code
@@ -283,3 +274,20 @@
                (- other-pointer-lowtag)
                (ash fdefn-raw-addr-slot word-shift))))
     (+ pc 5 (signed-sap-ref-32 (int-sap pc) 1)))) ; 5 = length of JMP
+
+;;; Undo the effects of XEP-ALLOCATE-FRAME
+;;; and point PC to FUNCTION
+(defun context-call-function (context function &optional arg-count)
+  (with-pinned-objects (function)
+    (let ((rsp (decf (context-register context rsp-offset) n-word-bytes))
+          (rbp (context-register context rbp-offset))
+          (fun-addr (get-lisp-obj-address function)))
+      (setf (sap-ref-word (int-sap rsp) 0)
+            (sap-ref-word (int-sap rbp) 8))
+      (when arg-count
+        (setf (context-register context rcx-offset)
+              (get-lisp-obj-address arg-count)))
+      (setf (context-register context rax-offset) fun-addr)
+      (set-context-pc context (sap-ref-word (int-sap fun-addr)
+                                            (- (ash simple-fun-self-slot word-shift)
+                                               fun-pointer-lowtag))))))
