@@ -47,11 +47,17 @@
   (or #+sbcl
       (let ((envvar (sb-ext:posix-getenv "SBCL_MAKE_PARALLEL")))
         (when envvar
+          (require :sb-posix)
           (parse-make-host-parallelism envvar)))))
 
 (defun make-host-1-parallelism () (car *make-host-parallelism*))
 (defun make-host-2-parallelism () (cdr *make-host-parallelism*))
 
+#+sbcl
+(let ((f (multiple-value-bind (sym access) (find-symbol "OS-EXIT" "SB-SYS")
+           (if (eq access :external) sym 'sb-unix:unix-exit))))
+  (defun exit-process (arg) (funcall f arg)))
+
 ;;; If TRUE, then COMPILE-FILE is being invoked only to process
 ;;; :COMPILE-TOPLEVEL forms, not to produce an output file.
 ;;; This is part of the implementation of parallelized make-host-2.
@@ -126,13 +132,51 @@
     (rename-file x path)))
 (compile 'rename-file-a-la-unix)
 
-;;; other miscellaneous tools
-(load "src/cold/read-from-file.lisp")
-(load "src/cold/rename-package-carefully.lisp")
-(load "src/cold/with-stuff.lisp")
+(export '(prepend-genfile-path read-from-file *generated-sources-root*))
+(defvar *generated-sources-root* "")
+
+;;; See remark in COMPILE-STEM about strings vs. The Common Lisp Way
+(defun prepend-genfile-path (namestring)
+  (concatenate 'string
+               ;; if exact match to "output/", or mismatch at the next character
+               (if (member (mismatch "output/" namestring) '(nil 7))
+                   *generated-sources-root*
+                   "")
+               namestring))
+(compile 'prepend-genfile-path) ; seems in vogue to compile everything in this file
+
+;;; Return an expression read from the file named NAMESTRING.
+;;; For user-supplied inputs, protect against more than one expression
+;;; appearing in the file. With trusted inputs we needn't bother.
+(defun read-from-file (namestring)
+  (with-open-file (s (prepend-genfile-path namestring))
+    (let* ((result (read s))
+           (eof-result (cons nil nil)))
+      (when (string= (pathname-name namestring) "build-order")
+        ;; build-order uses both host and target conditionals.
+        ;; Our sanity checks during make-host-1 would complain
+        ;; about #+feature when reading the second expression in the file
+        ;; whenever such feature is also a possible target feature.
+        (return-from read-from-file result))
+      (unless (eq (read s nil eof-result) eof-result)
+        (error "more than one expression in file ~S" namestring))
+      result)))
+(compile 'read-from-file)
 
 ;;; Try to minimize/conceal any non-standardness of the host Common Lisp.
-(load "src/cold/ansify.lisp")
+#-sbcl (load "src/cold/ansify.lisp")
+
+;;;; Do not put SBCL-specific things in 'ansify'. Put them here.
+;;;; And there had better not be a reason that SBCL needs ansification.
+#+sbcl
+(progn
+  (setq *compile-print* nil)
+  (load "src/cold/muffler.lisp")
+  ;; Let's just say we never care to see these.
+  (declaim (sb-ext:muffle-conditions
+            (satisfies unable-to-optimize-note-p)
+            (satisfies optional+key-style-warning-p)
+            sb-ext:code-deletion-note)))
 
 ;;;; special read-macros for building the cold system (and even for
 ;;;; building some of our tools for building the cold system)
@@ -142,9 +186,9 @@
 ;;; When cross-compiling, the *FEATURES* set for the target Lisp is
 ;;; not in general the same as the *FEATURES* set for the host Lisp.
 ;;; In order to refer to target features specifically, we refer to
-;;; *SHEBANG-FEATURES* instead of *FEATURES*, and use the #!+ and #!-
+;;; SB!XC:*FEATURES* instead of CL:*FEATURES*, and use the #!+ and #!-
 ;;; readmacros instead of the ordinary #+ and #- readmacros.
-(setf *shebang-features*
+(setf sb!xc:*features*
       (let* ((default-features
                (funcall (compile
                          nil
@@ -155,12 +199,7 @@
                              (compile nil
                                       (read-from-file customizer-file-name))
                              #'identity)))
-        (funcall customizer default-features)))
-(let ((*print-length* nil)
-      (*print-level* nil))
-  (format t
-          "target features *SHEBANG-FEATURES*=~%~@<~S~:>~%"
-          *shebang-features*))
+        (sort (funcall customizer default-features) #'string<)))
 
 (defvar *shebang-backend-subfeatures*
   (let* ((default-subfeatures nil)
@@ -170,18 +209,13 @@
                                   (read-from-file customizer-file-name))
                          #'identity)))
     (funcall customizer default-subfeatures)))
-(let ((*print-length* nil)
-      (*print-level* nil))
-  (format t
-          "target backend-subfeatures *SHEBANG-BACKEND-FEATURES*=~@<~S~:>~%"
-          *shebang-backend-subfeatures*))
 
 ;;; Call for effect of signaling an error if no target picked.
 (target-platform-name)
 
 ;;; You can get all the way through make-host-1 without either one of these
 ;;; features, but then 'bit-bash' will fail to cross-compile.
-(unless (intersection '(:big-endian :little-endian) *shebang-features*)
+(unless (intersection '(:big-endian :little-endian) sb!xc:*features*)
   (warn "You'll have bad time without either endian-ness defined"))
 
 ;;; Some feature combinations simply don't work, and sometimes don't
@@ -201,8 +235,14 @@
           ":GENCGC not supported on selected architecture")
          ("(not (or gencgc cheneygc))"
           "One of :GENCGC or :CHENEYGC must be enabled")
+         ("(and sb-safepoint (not (or arm64 ppc x86 x86-64)))"
+          ":SB-SAFEPOINT not supported on selected architecture")
+         ("(and sb-safepoint-strictly (not sb-safepoint))"
+          ":SB-SAFEPOINT-STRICTLY requires :SB-SAFEPOINT")
          ("(and sb-dynamic-core (not linkage-table))"
           ":SB-DYNAMIC-CORE requires :LINKAGE-TABLE")
+         ("(and relocatable-heap win32)"
+          "Relocatable heap requires (not win32)")
          ("(and sb-linkable-runtime (not sb-dynamic-core))"
           ":SB-LINKABLE-RUNTIME requires :SB-DYNAMIC-CORE")
          ("(and sb-linkable-runtime (not (or x86 x86-64)))"
@@ -226,7 +266,7 @@
           ":SB-THREAD not supported on selected architecture")))
       (failed-test-descriptions nil))
   (dolist (test feature-compatibility-tests)
-    (let ((*features* *shebang-features*))
+    (let ((cl:*features* sb!xc:*features*))
       (when (read-from-string (concatenate 'string "#+" (first test) "T NIL"))
         (push (second test) failed-test-descriptions))))
   (when failed-test-descriptions
@@ -251,7 +291,10 @@
 ;;; flags which can be used to describe properties of source files
 (defparameter
   *expected-stem-flags*
-  '(;; meaning: This file is not to be compiled when building the
+  '(;; meaning: This file is needed to generate C headers if doing so
+    ;; independently of make-host-1
+    :c-headers
+    ;; meaning: This file is not to be compiled when building the
     ;; cross-compiler which runs on the host ANSI Lisp. ("not host
     ;; code", i.e. does not execute on host -- but may still be
     ;; cross-compiled by the host, so that it executes on the target)
@@ -260,11 +303,6 @@
     ;; SBCL. ("not target code" -- but still presumably host code,
     ;; used to support the cross-compilation process)
     :not-target
-    ;; meaning: This file must always be compiled by 'slam.lisp' even if
-    ;; the object is not out of date with respect to its source.
-    ;; Necessary if there are compile-time-too effects that are not
-    ;; reflected into make-host-2 by load-time actions of make-host-1.
-    :slam-forcibly
     ;; meaning: The #'COMPILE-STEM argument :TRACE-FILE should be T.
     ;; When the compiler is SBCL's COMPILE-FILE or something like it,
     ;; compiling "foo.lisp" will generate "foo.trace" which contains lots
@@ -288,13 +326,11 @@
     ;; genesis to include in the cold core.
     :not-genesis))
 
-(defparameter *stems-and-flags* (read-from-file "build-order.lisp-expr"))
-
 (defvar *array-to-specialization* (make-hash-table :test #'eq))
 
 (defmacro do-stems-and-flags ((stem flags) &body body)
   (let ((stem-and-flags (gensym "STEM-AND-FLAGS")))
-    `(dolist (,stem-and-flags *stems-and-flags*)
+    `(dolist (,stem-and-flags (get-stems-and-flags))
        (let ((,stem (first ,stem-and-flags))
              (,flags (rest ,stem-and-flags)))
          ,@body
@@ -312,9 +348,15 @@
       stem)))
 (compile 'stem-remap-target)
 
-;;; Determine the source path for a stem.
+;;; Determine the source path for a stem by remapping from the abstract name
+;;; if it contains "/target/" and appending a ".lisp" suffix.
+;;; Assume that STEM is source-tree-relative unless it starts with "output/"
+;;; in which case it could be elsewhere, if you prefer to keep the sources
+;;; devoid of compilation artifacts. (The production of out-of-tree artifacts
+;;; is not actually implemented in the generic build, however if your build
+;;; system does that by itself, then hooray for you)
 (defun stem-source-path (stem)
-  (concatenate 'string "" (stem-remap-target stem) ".lisp"))
+  (concatenate 'string (prepend-genfile-path (stem-remap-target stem)) ".lisp"))
 (compile 'stem-source-path)
 
 ;;; Determine the object path for a stem/flags/mode combination.
@@ -337,8 +379,13 @@
     (concatenate 'string obj-prefix (stem-remap-target stem) obj-suffix)))
 (compile 'stem-object-path)
 
+(defvar *stems-and-flags* nil)
 ;;; Check for stupid typos in FLAGS list keywords.
-(let ((stems (make-hash-table :test 'equal)))
+(defun get-stems-and-flags ()
+ (when *stems-and-flags*
+   (return-from get-stems-and-flags *stems-and-flags*))
+ (setf *stems-and-flags* (read-from-file "build-order.lisp-expr"))
+ (let ((stems (make-hash-table :test 'equal)))
   (do-stems-and-flags (stem flags)
     ;; We do duplicate stem comparison based on the object path in
     ;; order to cover the case of stems with an :assem flag, which
@@ -358,6 +405,7 @@
       (when set-difference
         (error "found unexpected flag(s) in *STEMS-AND-FLAGS*: ~S"
                set-difference)))))
+  *stems-and-flags*)
 
 ;;;; tools to compile SBCL sources to create the cross-compiler
 
@@ -398,7 +446,7 @@
          (ignore-failure-p (find :ignore-failure-p flags)))
     (declare (type function compile-file))
 
-    (ensure-directories-exist obj :verbose t)
+    (ensure-directories-exist obj :verbose *compile-print*) ; host's value
 
     ;; We're about to set about building a new object file. First, we
     ;; delete any preexisting object file in order to avoid confusing
@@ -493,8 +541,7 @@
 (defun in-host-compilation-mode (fn)
   (declare (type function fn))
   (let ((*features* (cons :sb-xc-host *features*)))
-    (with-additional-nickname ("SB-XC" "SB!XC")
-      (funcall fn))))
+    (funcall fn)))
 (compile 'in-host-compilation-mode)
 
 ;;; Process a file as source code for the cross-compiler, compiling it
@@ -520,12 +567,24 @@
 ;;;; tools to compile SBCL sources to create object files which will
 ;;;; be used to create the target SBCL .core file
 
+(defun lpnify-stem (stem)
+  ;; Don't want genfiles path to sneak in - avoid (STEM-SOURCE-PATH ...) here.
+  (let ((string (stem-remap-target stem)))
+    ;; Distrust that random hosts don't bork up the translation.
+    ;; Simply replace '/' with ';' and be done.
+    (format nil "SYS:~:@(~A~).LISP" (substitute #\; #\/ string))))
+(compile 'lpnify-stem)
+
 ;;; Run the cross-compiler on a file in the source directory tree to
 ;;; produce a corresponding file in the target object directory tree.
 (defun target-compile-stem (stem flags)
   (funcall *in-target-compilation-mode-fn*
            (lambda ()
-             (compile-stem stem flags :target-compile))))
+             (progv (list (intern "*SOURCE-NAMESTRING*" "SB!C"))
+                    (list (lpnify-stem stem))
+               (loop
+                (with-simple-restart (recompile "Recompile")
+                  (return (compile-stem stem flags :target-compile))))))))
 (compile 'target-compile-stem)
 
 ;;; (This function is not used by the build process, but is intended
@@ -538,20 +597,3 @@
              (funcall *target-compile-file* filename))))
 (compile 'target-compile-file)
 
-(defun make-assembler-package (pkg-name)
-  (when (find-package pkg-name)
-    (delete-package pkg-name))
-  (let ((pkg (make-package pkg-name
-                           :use '("CL" "SB!INT" "SB!EXT" "SB!KERNEL" "SB!VM"
-                                  "SB!SYS" ; for SAP accessors
-                                  ;; Dependence of the assembler on the compiler
-                                  ;; feels a bit backwards, but assembly needs
-                                  ;; TN-SC, TN-OFFSET, etc. because the compiler
-                                  ;; doesn't speak the assembler's language.
-                                  ;; Rather vice-versa.
-                                  "SB!C"))))
-    ;; Both SB-ASSEM and SB-DISASSEM export these two symbols.
-    ;; Neither is shadowing-imported. If you need one, package-qualify it.
-    (shadow '("SEGMENT" "MAKE-SEGMENT") pkg)
-    (use-package '("SB!ASSEM" "SB!DISASSEM") pkg)
-    pkg))
