@@ -10,50 +10,40 @@
 ;;;; files for more information.
 
 (in-package "SB!VM")
-
-;;;; interfaces to IR2 conversion
-
-;;; Return a wired TN describing the N'th full call argument passing
-;;; location.
-(!def-vm-support-routine standard-arg-location (n)
-  (declare (type unsigned-byte n))
-  (if (< n register-arg-count)
-      (make-wired-tn *backend-t-primitive-type*
-                     register-arg-scn
-                     (elt *register-arg-offsets* n))
-      (make-wired-tn *backend-t-primitive-type*
-                     control-stack-arg-scn n)))
 
+(defconstant arg-count-sc (make-sc+offset immediate-arg-scn nargs-offset))
+(defconstant closure-sc (make-sc+offset descriptor-reg-sc-number lexenv-offset))
 
 ;;; Make a passing location TN for a local call return PC. If standard
 ;;; is true, then use the standard (full call) location, otherwise use
 ;;; any legal location. Even in the non-standard case, this may be
 ;;; restricted by a desire to use a subroutine call instruction.
-(!def-vm-support-routine make-return-pc-passing-location (standard)
+(defun make-return-pc-passing-location (standard)
   (if standard
-      (make-wired-tn *backend-t-primitive-type* register-arg-scn lra-offset)
-      (make-restricted-tn *backend-t-primitive-type* register-arg-scn)))
+      (make-wired-tn *backend-t-primitive-type* descriptor-reg-sc-number lra-offset)
+      (make-restricted-tn *backend-t-primitive-type* descriptor-reg-sc-number)))
 
 ;;; This is similar to MAKE-RETURN-PC-PASSING-LOCATION, but makes a
 ;;; location to pass OLD-FP in. This is (obviously) wired in the
 ;;; standard convention, but is totally unrestricted in non-standard
 ;;; conventions, since we can always fetch it off of the stack using
 ;;; the arg pointer.
-(!def-vm-support-routine make-old-fp-passing-location (standard)
-  (if standard
-      (make-wired-tn *fixnum-primitive-type* immediate-arg-scn ocfp-offset)
-      (make-normal-tn *fixnum-primitive-type*)))
+(defun make-old-fp-passing-location ()
+  (make-wired-tn *fixnum-primitive-type* immediate-arg-scn ocfp-offset))
+
+(defconstant old-fp-passing-offset
+  (make-sc+offset descriptor-reg-sc-number ocfp-offset))
 
 ;;; These functions make the TNs used to hold Old-FP and Return-PC
 ;;; within the current function. We treat these specially so that the
 ;;; debugger can find them at a known location.
-(!def-vm-support-routine make-old-fp-save-location (env)
+(defun make-old-fp-save-location (env)
   (specify-save-tn
    (physenv-debug-live-tn (make-normal-tn *fixnum-primitive-type*) env)
    (make-wired-tn *fixnum-primitive-type*
                   control-stack-arg-scn
                   ocfp-save-offset)))
-(!def-vm-support-routine make-return-pc-save-location (env)
+(defun make-return-pc-save-location (env)
   (let ((ptype *backend-t-primitive-type*))
     (specify-save-tn
      (physenv-debug-live-tn (make-normal-tn ptype) env)
@@ -62,48 +52,16 @@
 ;;; Make a TN for the standard argument count passing location. We
 ;;; only need to make the standard location, since a count is never
 ;;; passed when we are using non-standard conventions.
-(!def-vm-support-routine make-arg-count-location ()
+(defun make-arg-count-location ()
   (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nargs-offset))
-
-
-;;; Make a TN to hold the number-stack frame pointer. This is
-;;; allocated once per component, and is component-live.
-(!def-vm-support-routine make-nfp-tn ()
-  (component-live-tn
-   (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nfp-offset)))
-
-(!def-vm-support-routine make-stack-pointer-tn ()
-  (make-normal-tn *fixnum-primitive-type*))
-
-(!def-vm-support-routine make-number-stack-pointer-tn ()
-  (make-normal-tn *fixnum-primitive-type*))
-
-;;; Return a list of TNs that can be used to represent an
-;;; unknown-values continuation within a function.
-(!def-vm-support-routine make-unknown-values-locations ()
-  (list (make-stack-pointer-tn)
-        (make-normal-tn *fixnum-primitive-type*)))
-
-
-;;; This function is called by the ENTRY-ANALYZE phase, allowing
-;;; VM-dependent initialization of the IR2-COMPONENT structure. We
-;;; push placeholder entries in the CONSTANTS to leave room for
-;;; additional noise in the code object header.
-(!def-vm-support-routine select-component-format (component)
-  (declare (type component component))
-  (dotimes (i code-constants-offset)
-    (vector-push-extend nil
-                        (ir2-component-constants (component-info component))))
-  (values))
-
 
 ;;;; frame hackery
 
 ;;; Return the number of bytes needed for the current non-descriptor
-;;; stack frame. Non-descriptor stack frames must be multiples of 8
-;;; bytes on the PMAX.
+;;; stack frame. Non-descriptor stack frames must be multiples of 16
+;;; bytes to preserve alien stack alignment.
 (defun bytes-needed-for-non-descriptor-stack-frame ()
-  (* (logandc2 (1+ (sb-allocated-size 'non-descriptor-stack)) 1)
+  (* (logandc2 (+ 3 (sb-allocated-size 'non-descriptor-stack)) 3)
      n-word-bytes))
 
 ;;; This is used for setting up the Old-FP in local call.
@@ -121,18 +79,33 @@
   (:generator 1
     (let ((nfp (current-nfp-tn vop)))
       (when nfp
-        (inst addq nfp (bytes-needed-for-non-descriptor-stack-frame) val)))))
+        (inst lda val (bytes-needed-for-non-descriptor-stack-frame) nfp)))))
+
+;;; Accessing a slot from an earlier stack frame is definite hackery.
+(define-vop (ancestor-frame-ref)
+  (:args (frame-pointer :scs (descriptor-reg))
+         (variable-home-tn :load-if nil))
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:policy :fast-safe)
+  (:generator 4
+    (aver (sc-is variable-home-tn control-stack))
+    (loadw value frame-pointer (tn-offset variable-home-tn))))
+(define-vop (ancestor-frame-set)
+  (:args (frame-pointer :scs (descriptor-reg))
+         (value :scs (descriptor-reg any-reg)))
+  (:results (variable-home-tn :load-if nil))
+  (:policy :fast-safe)
+  (:generator 4
+    (aver (sc-is variable-home-tn control-stack))
+    (storew value frame-pointer (tn-offset variable-home-tn))))
 
 (define-vop (xep-allocate-frame)
-  (:info start-lab copy-more-arg-follows)
-  (:ignore copy-more-arg-follows)
-  (:vop-var vop)
+  (:info start-lab)
   (:temporary (:scs (non-descriptor-reg)) temp)
   (:generator 1
     ;; Make sure the function is aligned, and drop a label pointing to
     ;; this function header.
-    (align n-lowtag-bits)
-    (trace-table-entry trace-table-fun-prologue)
+    (emit-alignment n-lowtag-bits)
     (emit-label start-lab)
     ;; Allocate function header.
     (inst simple-fun-header-word)
@@ -145,35 +118,33 @@
       (inst compute-code-from-lip code-tn lip-tn entry-point temp)
       ;; ### We should also save it on the stack so that the garbage
       ;; collector won't forget about us if we call anyone else.
-      )
-    ;; Build our stack frames.
+      )))
+
+(define-vop (xep-setup-sp)
+  (:vop-var vop)
+  (:generator 1
     (inst lda
           csp-tn
           (* n-word-bytes (sb-allocated-size 'control-stack))
           cfp-tn)
     (let ((nfp (current-nfp-tn vop)))
       (when nfp
-        (inst subq nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)
-        (move nsp-tn nfp)))
-    (trace-table-entry trace-table-normal)))
+        (inst lda nsp-tn (- (bytes-needed-for-non-descriptor-stack-frame)) nsp-tn)
+        (move nsp-tn nfp)))))
 
 (define-vop (allocate-frame)
   (:results (res :scs (any-reg))
             (nfp :scs (any-reg)))
   (:info callee)
   (:generator 2
-    (trace-table-entry trace-table-fun-prologue)
     (move csp-tn res)
     (inst lda
           csp-tn
           (* n-word-bytes (sb-allocated-size 'control-stack))
           csp-tn)
     (when (ir2-physenv-number-stack-p callee)
-      (inst subq nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
-            nsp-tn)
-      (move nsp-tn nfp))
-    (trace-table-entry trace-table-normal)))
+      (inst lda nsp-tn (- (bytes-needed-for-non-descriptor-stack-frame)) nsp-tn)
+      (move nsp-tn nfp))))
 
 ;;; Allocate a partial frame for passing stack arguments in a full
 ;;; call. NARGS is the number of arguments passed. If no stack
@@ -392,6 +363,16 @@ default-value-8
               nvals)
   (:temporary (:scs (non-descriptor-reg)) temp))
 
+;;; This hook by the codegen lets us insert code before fall-thru entry points,
+;;; local-call entry points, and tail-call entry points.  The default does
+;;; nothing.
+(defun emit-block-header (start-label trampoline-label fall-thru-p alignp)
+  (declare (ignore fall-thru-p alignp))
+  (when trampoline-label
+    (emit-label trampoline-label))
+  (emit-label start-label))
+
+
 ;;;; local call with unknown values convention return
 
 ;;; Non-TR local call for a fixed number of values passed according to the
@@ -433,12 +414,10 @@ default-value-8
       (let ((callee-nfp (callee-nfp-tn callee)))
         (maybe-load-stack-nfp-tn callee-nfp nfp temp))
       (maybe-load-stack-tn cfp-tn fp)
-      (trace-table-entry trace-table-call-site)
       (inst compute-lra-from-code
             (callee-return-pc-tn callee) code-tn label temp)
       (note-this-location vop :call-site)
       (inst br zero-tn target)
-      (trace-table-entry trace-table-normal)
       (emit-return-pc label)
       (default-unknown-values vop values nvals move-temp temp label)
       (maybe-load-stack-nfp-tn cur-nfp nfp-save temp))))
@@ -470,12 +449,10 @@ default-value-8
       (let ((callee-nfp (callee-nfp-tn callee)))
         (maybe-load-stack-nfp-tn callee-nfp nfp temp))
       (maybe-load-stack-tn cfp-tn fp)
-      (trace-table-entry trace-table-call-site)
       (inst compute-lra-from-code
             (callee-return-pc-tn callee) code-tn label temp)
       (note-this-location vop :call-site)
       (inst bsr zero-tn target)
-      (trace-table-entry trace-table-normal)
       (emit-return-pc label)
       (note-this-location vop :unknown-return)
       (receive-unknown-values values-start nvals start count label temp)
@@ -510,12 +487,10 @@ default-value-8
       (let ((callee-nfp (callee-nfp-tn callee)))
         (maybe-load-stack-nfp-tn callee-nfp nfp temp))
       (maybe-load-stack-tn cfp-tn fp)
-      (trace-table-entry trace-table-call-site)
       (inst compute-lra-from-code
             (callee-return-pc-tn callee) code-tn label temp)
       (note-this-location vop :call-site)
       (inst bsr zero-tn target)
-      (trace-table-entry trace-table-normal)
       (emit-return-pc label)
       (note-this-location vop :known-return)
       (maybe-load-stack-nfp-tn cur-nfp nfp-save temp))))
@@ -540,18 +515,16 @@ default-value-8
   (:ignore val-locs vals)
   (:vop-var vop)
   (:generator 6
-    (trace-table-entry trace-table-fun-epilogue)
     (maybe-load-stack-tn ocfp-temp ocfp)
     (maybe-load-stack-tn return-pc-temp return-pc)
     (move cfp-tn csp-tn)
     (let ((cur-nfp (current-nfp-tn vop)))
       (when cur-nfp
-        (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)))
+        (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+              cur-nfp)))
     (inst subq return-pc-temp (- other-pointer-lowtag n-word-bytes) lip)
     (move ocfp-temp cfp-tn)
-    (inst ret zero-tn lip 1)
-    (trace-table-entry trace-table-normal)))
+    (inst ret zero-tn lip 1)))
 
 ;;;; full call:
 ;;;;
@@ -601,10 +574,12 @@ default-value-8
      (:args
       ,@(unless (eq return :tail)
           '((new-fp :scs (any-reg) :to :eval)))
-
-      ,(if named
-           '(name :target name-pass)
-           '(arg-fun :target lexenv))
+      ,@(case named
+          ((nil)
+           '((arg-fun :target lexenv)))
+          (:direct)
+          (t
+           '((name :target name-pass))))
 
       ,@(when (eq return :tail)
           '((ocfp :target ocfp-pass)
@@ -623,6 +598,7 @@ default-value-8
      (:vop-var vop)
      (:info ,@(unless (or variable (eq return :tail)) '(arg-locs))
             ,@(unless variable '(nargs))
+            ,@(when (eq named :direct) '(fun))
             ,@(when (eq return :fixed) '(nvals))
             step-instrumenting)
 
@@ -646,19 +622,20 @@ default-value-8
                   :to :eval)
                  return-pc-pass)
 
-     ,@(if named
-         `((:temporary (:sc descriptor-reg :offset fdefn-offset
-                        :from (:argument ,(if (eq return :tail) 0 1))
-                        :to :eval)
-                       name-pass))
-
-         `((:temporary (:sc descriptor-reg :offset lexenv-offset
-                        :from (:argument ,(if (eq return :tail) 0 1))
-                        :to :eval)
-                       lexenv)
-           #!-gengc
-           (:temporary (:scs (descriptor-reg) :from (:argument 0) :to :eval)
-                       function)))
+     ,@(case named
+         ((t)
+          `((:temporary (:sc descriptor-reg :offset fdefn-offset
+                         :from (:argument ,(if (eq return :tail) 0 1))
+                         :to :eval)
+                        name-pass)))
+         ((nil)
+          `((:temporary (:sc descriptor-reg :offset lexenv-offset
+                         :from (:argument ,(if (eq return :tail) 0 1))
+                         :to :eval)
+                        lexenv)
+            #!-gengc
+            (:temporary (:scs (descriptor-reg) :from (:argument 0) :to :eval)
+                        function))))
 
      (:temporary (:sc any-reg :offset nargs-offset :to :eval)
                  nargs-pass)
@@ -739,9 +716,9 @@ default-value-8
                                              word-shift)
                                          cfp-tn))))
                               (:frob-nfp
-                               (inst addq cur-nfp
+                               (inst lda nsp-tn
                                      (bytes-needed-for-non-descriptor-stack-frame)
-                                     nsp-tn)))
+                                     cur-nfp)))
                             `(#!-gengc
                               (:comp-lra
                                (inst compute-lra-from-code
@@ -755,53 +732,55 @@ default-value-8
                                     '(move new-fp cfp-tn)
                                     '(if (> nargs register-arg-count)
                                          (move new-fp cfp-tn)
-                                         (move csp-tn cfp-tn)))
-                               (trace-table-entry trace-table-call-site))))
+                                         (move csp-tn cfp-tn))))))
                       ((nil))))))
-
-           ,@(if named
-                 `((sc-case name
-                     (descriptor-reg (move name name-pass))
-                     (control-stack
-                      (inst ldl name-pass
-                            (ash (tn-offset name) word-shift) cfp-tn)
-                      (do-next-filler))
-                     (constant
-                      (inst ldl name-pass
-                            (- (ash (tn-offset name) word-shift)
-                               other-pointer-lowtag) code-tn)
-                      (do-next-filler)))
-                   (inst ldl entry-point
-                         (- (ash fdefn-raw-addr-slot word-shift)
-                            other-pointer-lowtag) name-pass)
-                   (do-next-filler))
-                 `((sc-case arg-fun
-                     (descriptor-reg (move arg-fun lexenv))
-                     (control-stack
-                      (inst ldl lexenv
-                            (ash (tn-offset arg-fun) word-shift) cfp-tn)
-                      (do-next-filler))
-                     (constant
-                      (inst ldl lexenv
-                            (- (ash (tn-offset arg-fun) word-shift)
-                               other-pointer-lowtag) code-tn)
-                      (do-next-filler)))
-                   #!-gengc
-                   (inst ldl function
-                         (- (ash closure-fun-slot word-shift)
-                            fun-pointer-lowtag) lexenv)
-                   #!-gengc
-                   (do-next-filler)
-                   #!-gengc
-                   (inst addq function
-                         (- (ash simple-fun-code-offset word-shift)
-                            fun-pointer-lowtag) entry-point)
-                   #!+gengc
-                   (inst ldl entry-point
-                         (- (ash closure-entry-point-slot word-shift)
-                            fun-pointer-lowtag) lexenv)
-                   #!+gengc
-                   (do-next-filler)))
+           ,@(case named
+               ((t)
+                `((sc-case name
+                    (descriptor-reg (move name name-pass))
+                    (control-stack
+                     (inst ldl name-pass
+                           (ash (tn-offset name) word-shift) cfp-tn)
+                     (do-next-filler))
+                    (constant
+                     (inst ldl name-pass
+                           (- (ash (tn-offset name) word-shift)
+                              other-pointer-lowtag) code-tn)
+                     (do-next-filler)))
+                  (inst ldl entry-point
+                        (- (ash fdefn-raw-addr-slot word-shift)
+                           other-pointer-lowtag) name-pass)
+                  (do-next-filler)))
+               ((nil)
+                `((sc-case arg-fun
+                    (descriptor-reg (move arg-fun lexenv))
+                    (control-stack
+                     (inst ldl lexenv
+                           (ash (tn-offset arg-fun) word-shift) cfp-tn)
+                     (do-next-filler))
+                    (constant
+                     (inst ldl lexenv
+                           (- (ash (tn-offset arg-fun) word-shift)
+                              other-pointer-lowtag) code-tn)
+                     (do-next-filler)))
+                  #!-gengc
+                  (inst ldl function
+                        (- (ash closure-fun-slot word-shift)
+                           fun-pointer-lowtag) lexenv)
+                  #!-gengc
+                  (do-next-filler)
+                  #!-gengc
+                  (inst addq function
+                        (- (ash simple-fun-code-offset word-shift)
+                           fun-pointer-lowtag) entry-point)
+                  #!+gengc
+                  (inst ldl entry-point
+                        (- (ash closure-entry-point-slot word-shift)
+                           fun-pointer-lowtag) lexenv)
+                  #!+gengc
+                  (do-next-filler)))
+               (:direct
+                `((inst ldl entry-point (static-fun-offset fun) null-tn))))
            (loop
              (if (cdr filler)
                  (do-next-filler)
@@ -813,26 +792,27 @@ default-value-8
 
          ,@(ecase return
              (:fixed
-              '((trace-table-entry trace-table-normal)
-                (emit-return-pc lra-label)
+              '((emit-return-pc lra-label)
                 (default-unknown-values vop values nvals
-                                        move-temp temp lra-label)
+                 move-temp temp lra-label)
                 (maybe-load-stack-nfp-tn cur-nfp nfp-save temp)))
              (:unknown
-              '((trace-table-entry trace-table-normal)
-                (emit-return-pc lra-label)
+              '((emit-return-pc lra-label)
                 (note-this-location vop :unknown-return)
                 (receive-unknown-values values-start nvals start count
-                                        lra-label temp)
+                 lra-label temp)
                 (maybe-load-stack-nfp-tn cur-nfp nfp-save temp)))
              (:tail))))))
 
 (define-full-call call nil :fixed nil)
 (define-full-call call-named t :fixed nil)
+(define-full-call static-call-named :direct :fixed nil)
 (define-full-call multiple-call nil :unknown nil)
 (define-full-call multiple-call-named t :unknown nil)
+(define-full-call static-multiple-call-named :direct :unknown nil)
 (define-full-call tail-call nil :tail nil)
 (define-full-call tail-call-named t :tail nil)
+(define-full-call static-tail-call-named :direct :tail nil)
 
 (define-full-call call-variable nil :fixed t)
 (define-full-call multiple-call-variable nil :unknown t)
@@ -866,8 +846,8 @@ default-value-8
     ;; Clear the number stack if anything is there.
     (let ((cur-nfp (current-nfp-tn vop)))
       (when cur-nfp
-        (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)))
+        (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+              cur-nfp)))
 
     ;; And jump to the assembly-routine that does the bliting.
     (inst li (make-fixup 'tail-call-variable :assembly-routine) temp)
@@ -888,11 +868,10 @@ default-value-8
   (:vop-var vop)
   (:generator 6
     ;; Clear the number stack.
-    (trace-table-entry trace-table-fun-epilogue)
     (let ((cur-nfp (current-nfp-tn vop)))
       (when cur-nfp
-        (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)))
+        (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+              cur-nfp)))
     ;; Clear the control stack, and restore the frame pointer.
     (move cfp-tn csp-tn)
     (move ocfp cfp-tn)
@@ -903,8 +882,7 @@ default-value-8
       (inst addq return-pc (* 2 n-word-bytes) temp)
       (unless (location= ra return-pc)
         (inst move ra return-pc))
-      (inst ret zero-tn temp 1))
-    (trace-table-entry trace-table-normal)))
+      (inst ret zero-tn temp 1))))
 
 ;;; Do unknown-values return of a fixed number of values. The Values
 ;;; are required to be set up in the standard passing locations. Nvals
@@ -939,11 +917,10 @@ default-value-8
   (:vop-var vop)
   (:generator 6
     ;; Clear the number stack.
-    (trace-table-entry trace-table-fun-epilogue)
     (let ((cur-nfp (current-nfp-tn vop)))
       (when cur-nfp
-        (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)))
+        (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+              cur-nfp)))
     ;; Establish the values pointer and values count.
     (move cfp-tn val-ptr)
     (inst li (fixnumize nvals) nargs)
@@ -961,8 +938,7 @@ default-value-8
       (dolist (reg (subseq (list a0 a1 a2 a3 a4 a5) nvals))
         (move null-tn reg)))
     ;; And away we go.
-    (lisp-return return-pc lip)
-    (trace-table-entry trace-table-normal)))
+    (lisp-return return-pc lip)))
 
 ;;; Do unknown-values return of an arbitrary number of values (passed
 ;;; on the stack.) We check for the common case of a single return
@@ -991,13 +967,12 @@ default-value-8
   (:vop-var vop)
 
   (:generator 13
-    (trace-table-entry trace-table-fun-epilogue)
     (let ((not-single (gen-label)))
       ;; Clear the number stack.
       (let ((cur-nfp (current-nfp-tn vop)))
         (when cur-nfp
-          (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-                nsp-tn)))
+          (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+                cur-nfp)))
 
       ;; Check for the single case.
       (inst li (fixnumize 1) a0)
@@ -1017,19 +992,10 @@ default-value-8
       (move vals-arg vals)
       (move nvals-arg nvals)
       (inst li (make-fixup 'return-multiple :assembly-routine) temp)
-      (inst jmp zero-tn temp))
-    (trace-table-entry trace-table-normal)))
+      (inst jmp zero-tn temp))))
 
 ;;;; XEP hackery
 
-;;; We don't need to do anything special for regular functions.
-(define-vop (setup-environment)
-  (:info label)
-  (:ignore label)
-  (:generator 0
-    ;; Don't bother doing anything.
-    ))
-
 ;;; Get the lexical environment from its passing location.
 (define-vop (setup-closure-environment)
   (:temporary (:sc descriptor-reg :offset lexenv-offset :target closure
@@ -1110,9 +1076,6 @@ default-value-8
 (define-full-reffer more-arg * 0 0 (descriptor-reg any-reg) * %more-arg)
 
 ;;; Turn &MORE arg (context, count) into a list.
-(defoptimizer (%listify-rest-args stack-allocate-result) ((&rest args))
-  t)
-
 (define-vop (listify-rest-args)
   (:args (context-arg :target context :scs (descriptor-reg))
          (count-arg :target count :scs (any-reg)))
@@ -1192,52 +1155,37 @@ default-value-8
     (inst subq supplied (fixnumize fixed) count)
     (inst subq csp-tn count context)))
 
-;;; Signal wrong argument count error if NARGS isn't equal to COUNT.
 (define-vop (verify-arg-count)
   (:policy :fast-safe)
-  (:translate sb!c::%verify-arg-count)
   (:args (nargs :scs (any-reg)))
-  (:arg-types positive-fixnum (:constant t))
-  (:temporary (:scs (any-reg) :type fixnum) temp)
-  (:info count)
+  (:arg-types positive-fixnum (:constant t) (:constant t))
+  (:info min max)
+  (:temporary (:scs (unsigned-reg)) temp)
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 3
     (let ((err-lab
-           (generate-error-code vop invalid-arg-count-error nargs)))
-      (cond ((zerop count)
-             (inst bne nargs err-lab))
-            (t
-             (inst subq nargs (fixnumize count) temp)
+            (generate-error-code vop 'invalid-arg-count-error nargs)))
+      (cond ((not min)
+             (cond ((zerop max)
+                    (inst bne nargs err-lab))
+                   (t
+                    (inst subq nargs (fixnumize max) temp)
+                    (inst bne temp err-lab))))
+            (max
+             (when (plusp min)
+               (inst li (fixnumize min) temp)
+               (inst cmpult nargs temp temp)
+               (inst bne temp err-lab))
+             (inst li (fixnumize max) temp)
+             (inst cmpult temp nargs temp)
+             (inst bne temp err-lab))
+            ((plusp min)
+             (inst li (fixnumize min) temp)
+             (inst cmpult nargs temp temp)
              (inst bne temp err-lab))))))
 
-;;; various other error signalers
-(macrolet ((frob (name error translate &rest args)
-             `(define-vop (,name)
-                ,@(when translate
-                    `((:policy :fast-safe)
-                      (:translate ,translate)))
-                (:args ,@(mapcar (lambda (arg)
-                                   `(,arg :scs (any-reg descriptor-reg)))
-                                 args))
-                (:vop-var vop)
-                (:save-p :compute-only)
-                (:generator 1000
-                  (error-call vop ,error ,@args)))))
-  (frob arg-count-error invalid-arg-count-error
-    sb!c::%arg-count-error nargs)
-  (frob type-check-error object-not-type-error sb!c::%type-check-error
-    object type)
-  (frob layout-invalid-error layout-invalid-error sb!c::%layout-invalid-error
-    object layout)
-  (frob odd-key-args-error odd-key-args-error
-    sb!c::%odd-key-args-error)
-  (frob unknown-key-arg-error unknown-key-arg-error
-    sb!c::%unknown-key-arg-error key)
-  (frob nil-fun-returned-error nil-fun-returned-error nil fun))
-
 ;;; Single-stepping
-
 (define-vop (step-instrument-before-vop)
   (:policy :fast-safe)
   (:vop-var vop)
