@@ -61,7 +61,7 @@
 ;;;             by APD on #lisp 2005-11-26: "MAYBE-INLINE lambda is
 ;;;             instantiated once per component, INLINE - for all
 ;;;             references (even under #'without FUNCALL)."
-(deftype inlinep ()
+(def!type inlinep ()
   '(member :inline :maybe-inline :notinline nil))
 (defconstant-eqx +inlinep-translations+
   '((inline . :inline)
@@ -69,6 +69,16 @@
     (maybe-inline . :maybe-inline))
   #'equal)
 
+(defstruct (dxable-args (:constructor make-dxable-args (list))
+                        (:predicate nil)
+                        (:copier nil))
+  (list nil :read-only t))
+(defstruct (inlining-data (:include dxable-args)
+                          (:constructor make-inlining-data (expansion list))
+                          (:predicate nil)
+                          (:copier nil))
+  (expansion nil :read-only t))
+
 ;;; *FREE-VARS* translates from the names of variables referenced
 ;;; globally to the LEAF structures for them. *FREE-FUNS* is like
 ;;; *FREE-VARS*, only it deals with function names.
@@ -179,7 +189,7 @@ the stack without triggering overflow protection.")
 (progn (declaim (fixnum *type-cache-nonce*))
        (!define-load-time-global *type-cache-nonce* 0))
 
-(def!struct (undefined-warning
+(defstruct (undefined-warning
             #-no-ansi-print-object
             (:print-object (lambda (x s)
                              (print-unreadable-object (x s :type t)
@@ -319,6 +329,29 @@ the stack without triggering overflow protection.")
   (declare (ignore thing))
   nil)
 
+;;; Various error-code generating helpers
+(defvar *adjustable-vectors*)
+
+(defmacro with-adjustable-vector ((var) &rest body)
+  `(let ((,var (or (pop *adjustable-vectors*)
+                   (make-array 16
+                               :element-type '(unsigned-byte 8)
+                               :fill-pointer 0
+                               :adjustable t))))
+     ;; Don't declare the length - if it gets adjusted and pushed back
+     ;; onto the freelist, it's anyone's guess whether it was expanded.
+     ;; This code was wrong for >12 years, so nobody must have needed
+     ;; more than 16 elements. Maybe we should make it nonadjustable?
+     (declare (type (vector (unsigned-byte 8)) ,var))
+     (setf (fill-pointer ,var) 0)
+     ;; No UNWIND-PROTECT here - semantics are unaffected by nonlocal exit,
+     ;; and this macro is about speeding up the compiler, not slowing it down.
+     ;; GC will clean up any debris, and since the vector does not point
+     ;; to anything, even an accidental promotion to a higher generation
+     ;; will not cause transitive garbage retention.
+     (prog1 (progn ,@body)
+       (push ,var *adjustable-vectors*))))
+
 (in-package "SB!ALIEN")
 
 ;;; Information describing a heap-allocated alien.
