@@ -210,6 +210,7 @@
                       (or #!+immobile-code (eq *compile-to-memory-space* :immobile))
                       box-num length)))
       (declare (type index box-num length))
+      (aver (<= box-num sb!vm:short-header-max-words))
 
       ;; The following operations need the code pinned:
       ;; 1. copying into code-instructions (a SAP)
@@ -235,6 +236,10 @@
             (setf (%simple-fun-name fun) (entry-info-name entry-info))
             (setf (%simple-fun-arglist fun) (entry-info-arguments entry-info))
             (setf (%simple-fun-type fun) (entry-info-type entry-info))
+            ;; We can't make a decision to record the source here
+            ;; because the policy is not available in a simple way.
+            ;; (It would demand groveling through the component to find
+            ;; the lambda-bind corresponding to this entry)
             (setf (%simple-fun-info fun) (entry-info-info entry-info))
             (note-fun entry-info fun object))))
 
@@ -265,8 +270,8 @@
 ;;; SOURCE-INFO list. We also check that there are no outstanding
 ;;; forward references to functions.
 (defun fix-core-source-info (info object &optional function)
-  (declare (type core-object object)
-           (type (or null function) function))
+  (declare (type core-object object))
+  (declare (type (or null function) function))
   (aver (zerop (hash-table-count (core-object-patch-table object))))
   (let ((source (debug-source-for-info info :function function)))
     (dolist (info (core-object-debug-info object))
