@@ -84,11 +84,12 @@
   (let ((block (tn-local tn))
         (num (tn-local-number tn)))
     (add-global-conflict
-     (if (zerop (sbit (ir2-block-written block) num))
-         :read-only
-         (if (zerop (sbit (ir2-block-live-out block) num))
-             :write
-             :read))
+     (cond ((zerop (sbit (ir2-block-written block) num))
+            :read-only)
+           ((zerop (sbit (ir2-block-live-out block) num))
+            :write)
+           (t
+            :read))
      tn block num))
   (values))
 
@@ -130,7 +131,7 @@
           (let* ((tn (tn-ref-tn ref))
                  (local (tn-local tn))
                  (kind (tn-kind tn)))
-            (unless (member kind '(:component :environment :constant))
+            (unless (member kind '(:component :environment :constant :unused))
               (unless (eq local block)
                 (when (= ltn-num local-tn-limit)
                   (return-from find-local-references vop))
@@ -310,22 +311,23 @@
              (tn-ref-across op)))
         ((null op))
       (let ((tn (tn-ref-tn op)))
-        (assert
-          (flet ((frob (refs)
-                   (do ((ref refs (tn-ref-next ref)))
-                       ((null ref) t)
-                     (when (and (eq (vop-block (tn-ref-vop ref)) block)
-                                (not (eq ref op)))
-                       (return nil)))))
-            (and (frob (tn-reads tn)) (frob (tn-writes tn))))
-          () "More operand ~S used more than once in its VOP." op)
-        (aver (not (find-in #'global-conflicts-next-blockwise tn
-                            (ir2-block-global-tns block)
-                            :key #'global-conflicts-tn)))
+        (unless (eq (tn-kind tn) :unused)
+          (assert
+           (flet ((frob (refs)
+                    (do ((ref refs (tn-ref-next ref)))
+                        ((null ref) t)
+                      (when (and (eq (vop-block (tn-ref-vop ref)) block)
+                                 (not (eq ref op)))
+                        (return nil)))))
+             (and (frob (tn-reads tn)) (frob (tn-writes tn))))
+           () "More operand ~S used more than once in its VOP." op)
+          (aver (not (find-in #'global-conflicts-next-blockwise tn
+                              (ir2-block-global-tns block)
+                              :key #'global-conflicts-tn)))
 
-        (add-global-conflict :read-only tn block num)
-        (setf (tn-local tn) block)
-        (setf (tn-local-number tn) num))))
+          (add-global-conflict :read-only tn block num)
+          (setf (tn-local tn) block)
+          (setf (tn-local-number tn) num)))))
   (values))
 
 (defevent coalesce-more-ltn-numbers
@@ -532,7 +534,7 @@
 ;;; for the read-only case, but switched at one point to always
 ;;; updating it. This generally speeds up the compiler nicely, but
 ;;; sometimes it causes an infinite loop in the updating machinery,
-;;; We cheat by switching of the fast path if it seems we're looping
+;;; We cheat by switching off the fast path if it seems we're looping
 ;;; longer then expected.
 (defun propagate-live-tns (block1 block2 fastp)
   (declare (type ir2-block block1 block2))
@@ -547,6 +549,13 @@
                 (tn-conflicts (tn-current-conflict tn))
                 (number1 (ir2-block-number block1)))
            (aver tn-conflicts)
+           (when (> (ir2-block-number (global-conflicts-block tn-conflicts))
+                    number1)
+             ;; The TN-CURRENT-CONFLICT finger overshot.  Reset it
+             ;; conservatively.
+             (setf tn-conflicts (tn-global-conflicts tn)
+                   (tn-current-conflict tn) tn-conflicts)
+             (aver tn-conflicts))
            (do ((current tn-conflicts (global-conflicts-next-tnwise current))
                 (prev nil current))
                ((or (null current)
@@ -576,6 +585,7 @@
 
 ;;; Do backward global flow analysis to find all TNs live at each
 ;;; block boundary.
+(defparameter *max-fast-propagate-live-tn-passes* 10)
 (defun lifetime-flow-analysis (component)
   ;; KLUDGE: This is the second part of the FASTP kludge in
   ;; propagate-live-tns: we pass fastp for ten first attempts,
@@ -596,13 +606,16 @@
 
           (dolist (b (block-succ block))
             (when (and (block-start b)
-                       (propagate-live-tns last (block-info b) (< i 10)))
+                       (propagate-live-tns
+                        last (block-info b)
+                        (< i *max-fast-propagate-live-tn-passes*)))
               (setq did-something t)))
 
           (do ((b (ir2-block-prev last) (ir2-block-prev b))
                (prev last b))
               ((not (eq (ir2-block-block b) block)))
-            (when (propagate-live-tns b prev (< i 10))
+            (when (propagate-live-tns b prev
+                                      (< i *max-fast-propagate-live-tn-passes*))
               (setq did-something t)))))
 
       (unless did-something (return))))
@@ -635,7 +648,8 @@
         (ecase (tn-kind tn)
           ((:normal :debug-environment)
            (setf (sbit live (tn-local-number tn)) 0))
-          (:environment :component))))
+          (:environment :component)
+          (:unused))))
     live))
 
 ;;; This is used to determine whether a :DEBUG-ENVIRONMENT TN should
@@ -665,7 +679,8 @@
                (not (eq (ir2-block-block next) 1block))
                (or (null succ)
                    (eq (first succ)
-                       (component-tail (block-component 1block)))))
+                       (component-tail (block-component 1block)))
+                   (block-tail-local-call-fun 1block)))
       (do ((conf (ir2-block-global-tns block)
                  (global-conflicts-next-blockwise conf)))
           ((null conf))
@@ -748,12 +763,6 @@
              (cached-block-physenv (ir2-block-block block))))))))
   (values))
 
-;;; FIXME: The next 3 macros aren't needed in the target runtime.
-;;; Figure out some way to make them only at build time. (Just
-;;; (EVAL-WHEN (:COMPILE-TOPLEVEL :EXECUTE) (DEFMACRO ..)) isn't good enough,
-;;; since we need CL:DEFMACRO at build-the-cross-compiler time and
-;;; SB!XC:DEFMACRO at run-the-cross-compiler time.)
-
 ;;; This is used in SCAN-VOP-REFS to simultaneously do something to
 ;;; all of the TNs referenced by a big more arg. We have to treat
 ;;; these TNs specially, since when we set or clear the bit in the
@@ -767,12 +776,14 @@
      (let ((prev ref))
        (do ((mref (tn-ref-next-ref ref) (tn-ref-next-ref mref)))
            ((null mref))
-         (let ((mtn (tn-ref-tn mref)))
-           (unless (eql (tn-local-number mtn) num)
-             (return))
-           ,action)
-         (setq prev mref))
-       (setq ref prev))))
+         (let* ((mtn (tn-ref-tn mref))
+                (tn-number (tn-local-number mtn)))
+           (when tn-number
+             (unless (eql tn-number num)
+               (return))
+             ,action
+             (setq prev mref)
+             (setq ref prev)))))))
 
 ;;; Handle the part of CONFLICT-ANALYZE-1-BLOCK that scans the REFs
 ;;; for the current VOP. This macro shamelessly references free
