@@ -12,6 +12,9 @@
 
 (in-package "SB!C")
 
+(declaim (type fixnum *compiler-sset-counter*))
+(defvar *compiler-sset-counter* 0)
+
 ;;; The front-end data structure (IR1) is composed of nodes,
 ;;; representing actual evaluations. Linear sequences of nodes in
 ;;; control-flow order are combined into blocks (but see
@@ -34,7 +37,7 @@
 ;;; that works only for compiling the compiler.
 
 ;;; "Lead-in" Control TRANsfer [to some node]
-(def!struct (ctran (:constructor make-ctran))
+(def!struct (ctran (:constructor make-ctran) (:copier nil))
   ;; an indication of the way that this continuation is currently used
   ;;
   ;; :UNUSED
@@ -69,7 +72,8 @@
 
 ;;; Linear VARiable. Multiple-value (possibly of unknown number)
 ;;; temporal storage.
-(def!struct (lvar (:constructor make-lvar (&optional dest)))
+(def!struct (lvar (:constructor make-lvar (&optional dest))
+                  (:copier nil))
   ;; The node which receives this value. NIL only temporarily.
   (dest nil :type (or node null))
   ;; cached type of this lvar's value. If NIL, then this must be
@@ -89,7 +93,8 @@
   ;; if the LVAR value is DYNAMIC-EXTENT, CLEANUP protecting it.
   (dynamic-extent nil :type (or null cleanup))
   ;; something or other that the back end annotates this lvar with
-  (info nil))
+  (info nil)
+  (dependent-casts nil))
 (!set-load-form-method lvar (:xc :target) :ignore-it)
 
 (defmethod print-object ((x lvar) stream)
@@ -119,7 +124,9 @@
   (setf (lvar-%externally-checkable-type lvar) nil))
 
 (def!struct (node (:constructor nil)
-                  (:include sset-element (number (incf *compiler-sset-counter*)))
+                  (:include sset-element
+                            (number (unless (eql *compiler-sset-counter* 0)
+                                      (incf *compiler-sset-counter*))))
                   (:copier nil))
   ;; unique ID for debugging
   #!+sb-show (id (new-object-id) :read-only t)
@@ -176,6 +183,7 @@
 
 (defun %with-ir1-environment-from-node (node fun)
   (declare (type node node) (type function fun))
+  #-sb-xc-host (declare (dynamic-extent fun)) ; "unable"
   (let ((*current-component* (node-component node))
         (*lexenv* (node-lexenv node))
         (*current-path* (node-source-path node)))
@@ -213,14 +221,8 @@
 ;;;    set, since they may:
 ;;;     - be in the process of being deleted, or
 ;;;     - have no successors.
-;;; -- TYPE-ASSERTED, TEST-MODIFIED
-;;;    These flags are used to indicate that something in this block
-;;;    might be of interest to constraint propagation. TYPE-ASSERTED
-;;;    is set when an lvar type assertion is strengthened.
-;;;    TEST-MODIFIED is set whenever the test for the ending IF has
-;;;    changed (may be true when there is no IF.)
 (!def-boolean-attribute block
-  reoptimize flush-p type-check delete-p type-asserted test-modified)
+  reoptimize flush-p type-check delete-p)
 
 (macrolet ((defattr (block-slot)
              `(defmacro ,block-slot (block)
@@ -230,14 +232,12 @@
   (defattr block-reoptimize)
   (defattr block-flush-p)
   (defattr block-type-check)
-  (defattr block-delete-p)
-  (defattr block-type-asserted)
-  (defattr block-test-modified))
+  (defattr block-delete-p))
 
 (def!struct (cloop (:conc-name loop-)
                    (:predicate loop-p)
                    (:constructor make-loop)
-                   (:copier copy-loop))
+                   (:copier nil))
   ;; The kind of loop that this is.  These values are legal:
   ;;
   ;;    :OUTER
@@ -286,6 +286,7 @@
 (def!struct (cblock (:include sset-element)
                     (:constructor make-block (start))
                     (:constructor make-block-key)
+                    (:copier nil)
                     (:conc-name block-)
                     (:predicate block-p))
   ;; a list of all the blocks that are predecessors/successors of this
@@ -309,8 +310,7 @@
   (next nil :type (or null cblock))
   (prev nil :type (or null cblock))
   ;; This block's attributes: see above.
-  (flags (block-attributes reoptimize flush-p type-check type-asserted
-                           test-modified)
+  (flags (block-attributes reoptimize flush-p type-check)
          :type attributes)
   ;; in constraint propagation: list of LAMBDA-VARs killed in this block
   ;; in copy propagation: list of killed TNs
@@ -536,7 +536,8 @@
   ;; the kind of thing that has to be cleaned up
   (kind (missing-arg)
         :type (member :special-bind :catch :unwind-protect
-                      :block :tagbody :dynamic-extent))
+                      :block :tagbody :dynamic-extent
+                      #!-c-stack-is-control-stack :restore-nsp))
   ;; the node that messes things up. This is the last node in the
   ;; non-messed-up environment. Null only temporarily. This could be
   ;; deleted due to unreachability.
@@ -632,6 +633,7 @@
 ;;; continuation, although it is accessed by searching in the
 ;;; PHYSENV-NLX-INFO.
 (def!struct (nlx-info
+             (:copier nil)
              (:constructor make-nlx-info (cleanup
                                           exit
                                           &aux
@@ -677,7 +679,10 @@
 ;;; structures. A reference to a LEAF is indicated by a REF node. This
 ;;; allows us to easily substitute one for the other without actually
 ;;; hacking the flow graph.
-(def!struct (leaf (:include sset-element (number (incf *compiler-sset-counter*)))
+(def!struct (leaf (:include sset-element
+                            (number (unless (eql *compiler-sset-counter* 0)
+                                      (incf *compiler-sset-counter*))))
+                  (:copier nil)
                   (:constructor nil))
   ;; unique ID for debugging
   #!+sb-show (id (new-object-id) :read-only t)
@@ -736,8 +741,6 @@
       extent)))
 
 ;;; LEAF name operations
-;;;
-;;; KLUDGE: wants CLOS..
 (defun leaf-has-source-name-p (leaf)
   (not (eq (leaf-%source-name leaf)
            '.anonymous.)))
@@ -748,17 +751,28 @@
 ;;; The BASIC-VAR structure represents information common to all
 ;;; variables which don't correspond to known local functions.
 (def!struct (basic-var (:include leaf)
+                       (:copier nil)
                        (:constructor nil))
   ;; Lists of the set nodes for this variable.
   (sets () :type list))
 
 ;;; The GLOBAL-VAR structure represents a value hung off of the symbol
 ;;; NAME.
-(def!struct (global-var (:include basic-var))
+(def!struct (global-var (:include basic-var) (:copier nil))
   ;; kind of variable described
   (kind (missing-arg)
         :type (member :special :global-function :global :unknown)))
-(defprinter (global-var :identity t)
+
+(defun pretty-print-global-var (var stream)
+  (let ((name (leaf-source-name var)))
+    (princ (if (eq (global-var-kind var) :global-function)
+               `(function ,name)
+               name)
+           stream)))
+
+(defprinter (global-var :identity t
+             :pretty-ir-printer
+             (pretty-print-global-var structure stream))
   %source-name
   #!+sb-show id
   (type :test (not (eq type *universal-type*)))
@@ -783,7 +797,8 @@
 ;;; INLINEP values are preserved.
 (def!struct (defined-fun (:include global-var
                                    (where-from :defined)
-                                   (kind :global-function)))
+                                   (kind :global-function))
+                         (:copier nil))
   ;; The values of INLINEP and INLINE-EXPANSION initialized from the
   ;; global environment.
   (inlinep nil :type inlinep)
@@ -793,7 +808,8 @@
   ;; RECOGNIZE-KNOWN-CALL) - we need separate functionals for each policy in
   ;; which the function is used.
   (functionals nil :type list))
-(defprinter (defined-fun :identity t)
+(defprinter (defined-fun :identity t
+             :pretty-ir-printer (pretty-print-global-var structure stream))
   %source-name
   #!+sb-show id
   inlinep
@@ -807,7 +823,8 @@
 (def!struct (functional (:include leaf
                                   (%source-name '.anonymous.)
                                   (where-from :defined)
-                                  (type (specifier-type 'function))))
+                                  (type (specifier-type 'function)))
+                        (:copier nil))
   ;; (For public access to this slot, use LEAF-DEBUG-NAME.)
   ;;
   ;; the name of FUNCTIONAL for debugging purposes, or NIL if we
@@ -953,7 +970,17 @@
   ;; True if this functional was created from an inline expansion. This
   ;; is either T, or the GLOBAL-VAR for which it is an expansion.
   (inline-expanded nil))
-(defprinter (functional :identity t)
+
+(defun pretty-print-functional (functional stream)
+  (let ((name (functional-debug-name functional)))
+    (princ `(function
+             ,(if (typep name '(cons (member xep tl-xep)))
+                  (cadr name)
+                  name))
+           stream)))
+
+(defprinter (functional :identity t
+             :pretty-ir-printer (pretty-print-functional structure stream))
   %source-name
   %debug-name
   #!+sb-show id)
@@ -1011,7 +1038,7 @@
                      (:conc-name lambda-)
                      (:predicate lambda-p)
                      (:constructor make-lambda)
-                     (:copier copy-lambda))
+                     (:copier nil))
   ;; list of LAMBDA-VAR descriptors for arguments
   (vars nil :type list :read-only t)
   ;; If this function was ever a :OPTIONAL function (an entry-point
@@ -1073,7 +1100,8 @@
   ;; the lambda itself is not, and the bindings introduced by it are considered
   ;; transparent by the nested DX analysis.
   (system-lambda-p nil :type boolean))
-(defprinter (clambda :conc-name lambda- :identity t)
+(defprinter (clambda :conc-name lambda- :identity t
+             :pretty-ir-printer (pretty-print-functional structure stream))
   %source-name
   %debug-name
   #!+sb-show id
@@ -1124,7 +1152,7 @@
 ;;; arguments into a direct call to the appropriate entry-point
 ;;; function, so functions that are compiled together can avoid doing
 ;;; the dispatch.
-(def!struct (optional-dispatch (:include functional))
+(def!struct (optional-dispatch (:include functional) (:copier nil))
   ;; the original parsed argument list, for anyone who cares
   (arglist nil :type list)
   ;; true if &ALLOW-OTHER-KEYS was supplied
@@ -1154,7 +1182,8 @@
   ;; be used by callers that supply at least MAX-ARGS arguments and
   ;; know what they are doing.
   (main-entry nil :type (or clambda null)))
-(defprinter (optional-dispatch :identity t)
+(defprinter (optional-dispatch :identity t
+             :pretty-ir-printer (pretty-print-functional structure stream))
   %source-name
   %debug-name
   #!+sb-show id
@@ -1173,7 +1202,7 @@
 ;;; LAMBDA-VARs during IR1 conversion. If we use one of these things,
 ;;; then the var will have to be massaged a bit before it is simple
 ;;; and lexical.
-(def!struct arg-info
+(def!struct (arg-info (:copier nil))
   ;; true if this arg is to be specially bound
   (specialp nil :type boolean)
   ;; the kind of argument being described. Required args only have arg
@@ -1232,7 +1261,7 @@
   explicit-value-cell
   )
 
-(def!struct (lambda-var (:include basic-var))
+(def!struct (lambda-var (:include basic-var) (:copier nil))
   (flags (lambda-var-attributes)
          :type attributes)
   ;; the CLAMBDA that this var belongs to. This may be null when we are
@@ -1313,7 +1342,7 @@
                  (:conc-name if-)
                  (:predicate if-p)
                  (:constructor make-if)
-                 (:copier copy-if))
+                 (:copier nil))
   ;; LVAR for the predicate
   (test (missing-arg) :type lvar)
   ;; the blocks that we execute next in true and false case,
@@ -1333,7 +1362,7 @@
                   (:conc-name set-)
                   (:predicate set-p)
                   (:constructor make-set)
-                  (:copier copy-set))
+                  (:copier nil))
   ;; descriptor for the variable set
   (var (missing-arg) :type basic-var)
   ;; LVAR for the value form
@@ -1364,12 +1393,14 @@
   ;; NOTINLINE. :ERROR is like :FULL, but means that we have
   ;; discovered that the call contains an error, and should not be
   ;; reconsidered for optimization.
-  (kind :full :type (member :local :full :error :known))
+  (kind :full :type (member :local :full :error :known
+                            :unknown-keys))
   ;; if a call to a known global function, contains the FUN-INFO.
   (fun-info nil :type (or fun-info null))
   ;; Untrusted type we have asserted for this combination.
   (type-validated-for-leaf nil)
   ;; some kind of information attached to this node by the back end
+  ;; or by CHECK-IMPORTANT-RESULT
   (info nil)
   (step-info))
 
@@ -1416,7 +1447,7 @@
                      (:conc-name return-)
                      (:predicate return-p)
                      (:constructor make-return)
-                     (:copier copy-return))
+                     (:copier nil))
   ;; the lambda we are returning from. Null temporarily during
   ;; ir1tran.
   (lambda nil :type (or clambda null))
@@ -1435,6 +1466,7 @@
 ;;; TYPE-TO-CHECK is performed and then the VALUE is declared to be of
 ;;; type ASSERTED-TYPE.
 (def!struct (cast (:include valued-node)
+                  (:copier nil)
                   (:constructor %make-cast))
   (asserted-type (missing-arg) :type ctype)
   (type-to-check (missing-arg) :type ctype)
@@ -1450,13 +1482,6 @@
   ;; T
   ;;    A type check is needed.
   (%type-check t :type (member t :external nil))
-  ;; the LEXENV for the deleted EXIT node for which this is the
-  ;; remaining value semantics. If NULL, we do not have exit value
-  ;; semantics and may be deleted based on type information.
-  (vestigial-exit-lexenv nil :type (or lexenv null))
-  ;; the LEXENV for the ENTRY node for the deleted EXIT node mentioned
-  ;; above. NULL if we do not have exit value semantics.
-  (vestigial-exit-entry-lexenv nil :type (or lexenv null))
   ;; the lvar which is checked
   (value (missing-arg) :type lvar)
   (context nil)
@@ -1467,9 +1492,14 @@
   %type-check
   value
   asserted-type
-  type-to-check
-  vestigial-exit-lexenv
-  vestigial-exit-entry-lexenv)
+  type-to-check)
+
+;;; A filter to help order the value semantics of MULTIPLE-VALUE-PROG1
+(def!struct (vestigial-exit-cast (:include cast
+                                           (%type-check nil)
+                                           (asserted-type *wild-type*)
+                                           (type-to-check *wild-type*))
+                                 (:copier nil)))
 
 ;;; A cast that always follows %check-bound and they are deleted together.
 ;;; Created via BOUND-CAST ir1-translator by chaining it together with %check-bound.
@@ -1477,7 +1507,8 @@
 ;;; DELETE-CAST deletes BOUND-CAST-CHECK
 ;;; GENERATE-TYPE-CHECKS ignores it, it never translates to a type check,
 ;;; %CHECK-BOUND does all the checking.
-(def!struct (bound-cast (:include cast (%type-check nil)))
+(def!struct (bound-cast (:include cast (%type-check nil))
+                        (:copier nil))
   ;; %check-bound combination before the cast
   (check (missing-arg) :type (or null combination))
   ;; Tells whether the type information is in a state where it can be
@@ -1486,14 +1517,32 @@
   (array (missing-arg) :type lvar)
   (bound (missing-arg) :type lvar))
 
-;;; Used for marking CALLABLE arguments with unrecognizable LVARS in
-;;; VALID-CALLABLE-ARGUMENT so that it can be rerun in
-;;; IR1-OPTIMIZE-CAST with better information.
-;;; Also used by CALLABLE-CAST
-(def!struct (function-designator-cast (:include cast))
-  (arg-count (missing-arg) :type index)
-  (caller nil :type symbol))
+(def!struct (dependent-cast (:include cast) (:copier nil))
+  ;; Either LVARs or LEAFs
+  (deps nil :type list))
 
+(def!struct (function-designator-cast (:include dependent-cast)
+                                      (:copier nil))
+  (caller nil :type symbol)
+  (arg-specs nil :type list)
+  (result-specs nil :type list))
+(defprinter (function-designator-cast :identity t)
+  %type-check
+  value
+  asserted-type
+  (deps :test deps)
+  (caller :test caller)
+  (arg-specs :test arg-specs)
+  (result-specs :test result-specs))
+
+;;; Something will modify this value, warn if it's a constant
+(def!struct (modifying-cast (:include cast) (:copier nil))
+  ;; NIL after a warning has been issued
+  caller)
+
+;;; The hook is called when the value becomes constant
+(def!struct (cast-with-hook (:include cast) (:copier nil))
+  (hook #'missing-arg :type (or null function)))
 
 ;;;; non-local exit support
 ;;;;
