@@ -104,28 +104,24 @@
 
 (!gencgc-space-setup #x20000000
                      :dynamic-space-start #x1000000000
-
-                     #!+openbsd :default-dynamic-space-size #!+openbsd #x1bcf0000
-
-                     #!+win32 :alignment #!+win32 #x10000)
+                     #!+openbsd :dynamic-space-size #!+openbsd #x1bcf0000)
 
 (defconstant linkage-table-entry-size 16)
 
 
-;;;; other miscellaneous constants
-
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
   single-step-around-trap
   single-step-before-trap
   invalid-arg-count-trap
+  memory-fault-emulation-trap
   #!+sb-safepoint global-safepoint-trap
-  #!+sb-safepoint csp-safepoint-trap)
+  #!+sb-safepoint csp-safepoint-trap
+  error-trap)
 
 ;;;; static symbols
 
@@ -140,34 +136,21 @@
 ;;; we could profitably keep these in registers on x86-64 now we have
 ;;; r8-r15 as well
 ;;;     Note these spaces grow from low to high addresses.
-(defvar *allocation-pointer*)
 (defvar *binding-stack-pointer*)
 
 (defconstant-eqx +static-symbols+
-  `(,@+common-static-symbols+
-    ,@+c-callable-static-symbols+
-    *alien-stack-pointer*
-
+ `#(,@+common-static-symbols+
+    #!+(and immobile-space (not sb-thread)) function-layout
+    #!-sb-thread *alien-stack-pointer*    ; a thread slot if #!+sb-thread
      ;; interrupt handling
-     *pseudo-atomic-bits*
-
-     *allocation-pointer*
-     *binding-stack-pointer*
-
-     ;; For GC-AND-SAVE
-     *restart-lisp-function*
-
-     ;; Needed for callbacks to work across saving cores. see
-     ;; ALIEN-CALLBACK-ASSEMBLER-WRAPPER in c-call.lisp for gory
-     ;; details.
-     sb!alien::*enter-alien-callback*
-     )
-  #'equal)
+    #!-sb-thread *pseudo-atomic-bits*     ; ditto
+    #!-sb-thread *binding-stack-pointer*) ; ditto
+  #'equalp)
 
 ;;; FIXME: with #!+immobile-space, this should be the empty list,
 ;;; because *all* fdefns are permanently placed.
 (defconstant-eqx +static-fdefns+
-  '(length
+  #(length
     two-arg-+
     two-arg--
     two-arg-*
@@ -183,7 +166,7 @@
     two-arg-gcd
     two-arg-lcm
     %coerce-callable-to-fun)
-  #'equal)
+  #'equalp)
 
 #!+sb-simd-pack
 (defvar *simd-pack-element-types* '(integer single-float double-float))
