@@ -26,13 +26,9 @@
 #include "parse.h"
 #include "vars.h"
 
-/* Almost all of this file can be skipped if we're not supporting LDB. */
-#if defined(LISP_FEATURE_SB_LDB)
-
 #include "print.h"
 #include "arch.h"
 #include "interr.h"
-#include "gc.h"
 #include "search.h"
 #include "purify.h"
 #include "globals.h"
@@ -147,7 +143,7 @@ dump_cmd(char **ptr)
 #else
         printf("0x%08X: ", (u32) addr);
 #endif
-        if (force || is_valid_lisp_addr((os_vm_address_t)addr)) {
+        if (force || gc_managed_addr_p((lispobj)addr)) {
 #ifndef LISP_FEATURE_ALPHA
             unsigned long *lptr = (unsigned long *)addr;
 #else
@@ -219,14 +215,12 @@ regs_cmd(char **ptr)
 #ifdef LISP_FEATURE_GENCGC
     /* printf("DYNAMIC\t=\t%p\n", (void*)DYNAMIC_SPACE_START); */
 #else
-    printf("STATIC\t=\t%p   ",
-           (void*)SymbolValue(STATIC_SPACE_FREE_POINTER, thread));
-    printf("RDONLY\t=\t%p   ",
-           (void*)SymbolValue(READ_ONLY_SPACE_FREE_POINTER, thread));
+    printf("STATIC\t=\t%p   ", static_space_free_pointer);
+    printf("RDONLY\t=\t%p   ", read_only_space_free_pointer);
     printf("DYNAMIC\t=\t%p\n", (void*)current_dynamic_space);
 #endif
 
-#ifdef reg_ALLOC
+#ifndef ALLOCATION_POINTER
     printf("ALLOC\t=\t%p\n", (void*)dynamic_space_free_pointer);
 #else
     printf("ALLOC\t=\t%p\n", (void*)SymbolValue(ALLOCATION_POINTER, thread));
@@ -361,7 +355,7 @@ print_context(os_context_t *context)
         brief_print((lispobj)(*os_context_register_addr(context,i)));
 #endif
     }
-#ifdef LISP_FEATURE_DARWIN
+#if defined(LISP_FEATURE_DARWIN) && defined(LISP_FEATURE_PPC)
     printf("DAR:\t\t 0x%08lx\n", (unsigned long)(*os_context_register_addr(context, 41)));
     printf("DSISR:\t\t 0x%08lx\n", (unsigned long)(*os_context_register_addr(context, 42)));
 #endif
@@ -375,7 +369,7 @@ print_context_cmd(char **ptr)
     int free_ici;
     struct thread *thread=arch_os_get_current_thread();
 
-    free_ici = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,thread));
+    free_ici = fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,thread));
 
     if (more_p(ptr)) {
         int index;
@@ -385,7 +379,7 @@ print_context_cmd(char **ptr)
         if ((index >= 0) && (index < free_ici)) {
             printf("There are %d interrupt contexts.\n", free_ici);
             printf("printing context %d\n", index);
-            print_context(thread->interrupt_contexts[index]);
+            print_context(nth_interrupt_context(index, thread));
         } else {
             printf("There aren't that many/few contexts.\n");
             printf("There are %d interrupt contexts.\n", free_ici);
@@ -396,7 +390,7 @@ print_context_cmd(char **ptr)
         else {
             printf("There are %d interrupt contexts.\n", free_ici);
             printf("printing context %d\n", free_ici - 1);
-            print_context(thread->interrupt_contexts[free_ici - 1]);
+            print_context(nth_interrupt_context(free_ici - 1, thread));
         }
     }
 }
@@ -419,10 +413,8 @@ backtrace_cmd(char **ptr)
 static void
 catchers_cmd(char **ptr)
 {
-    struct catch_block *catch;
-    struct thread *thread=arch_os_get_current_thread();
-
-    catch = (struct catch_block *)SymbolValue(CURRENT_CATCH_BLOCK,thread);
+    struct catch_block *catch = (struct catch_block *)
+        read_TLS(CURRENT_CATCH_BLOCK, arch_os_get_current_thread());
 
     if (catch == NULL)
         printf("There are no active catchers!\n");
@@ -536,20 +528,9 @@ throw_to_monitor()
     longjmp(curbuf, 1);
 }
 
-#endif /* defined(LISP_FEATURE_SB_LDB) */
-
 /* what we do when things go badly wrong at a low level */
 void
 monitor_or_something()
 {
-#if defined(LISP_FEATURE_SB_LDB)
     ldb_monitor();
-#else
-     fprintf(stderr,
-"The system is too badly corrupted or confused to continue at the Lisp\n\
-level. If the system had been compiled with the SB-LDB feature, we'd drop\n\
-into the LDB low-level debugger now. But there's no LDB in this build, so\n\
-we can't really do anything but just exit, sorry.\n");
-    exit(1);
-#endif
 }
