@@ -67,11 +67,15 @@ typedef enum {
 
 void map_gc_page();
 void unmap_gc_page();
-int check_pending_interrupts();
 void gc_state_lock();
 void gc_state_wait(gc_phase_t);
+int gc_cycle_active(void);
 void gc_state_unlock();
 
+#define WITH_GC_STATE_LOCK \
+    gc_state_lock(); \
+    RUN_BODY_ONCE(gc_state_lock, gc_state_unlock())
+
 #endif
 
 /*
@@ -271,6 +275,20 @@ HeaderValue(lispobj obj)
   return obj >> N_WIDETAG_BITS;
 }
 
+static inline int listp(lispobj obj) {
+    return lowtag_of(obj) == LIST_POINTER_LOWTAG;
+}
+static inline int instancep(lispobj obj) {
+    return lowtag_of(obj) == INSTANCE_POINTER_LOWTAG;
+}
+static inline int functionp(lispobj obj) {
+    return lowtag_of(obj) == FUN_POINTER_LOWTAG;
+}
+static inline int simple_vector_p(lispobj obj) {
+    return lowtag_of(obj) == OTHER_POINTER_LOWTAG &&
+           widetag_of(*(lispobj*)(obj-OTHER_POINTER_LOWTAG)) == SIMPLE_VECTOR_WIDETAG;
+}
+
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
 #define HEADER_VALUE_MASKED(x) (HeaderValue(x) & SHORT_HEADER_MAX_WORDS)
 #else
@@ -280,38 +298,15 @@ static inline uword_t instance_length(lispobj header)
 {
   return HEADER_VALUE_MASKED(header);
 }
-static inline lispobj instance_layout(lispobj* instance_ptr) // native ptr
-{
-#ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
-  return instance_ptr[0] >> 32; // the high half of the header is the layout
+/* Define an assignable instance_layout() macro taking a native pointer */
+#ifndef LISP_FEATURE_COMPACT_INSTANCE_HEADER
+# define instance_layout(instance_ptr) (instance_ptr)[1]
+#elif defined(LISP_FEATURE_64_BIT) && defined(LISP_FEATURE_LITTLE_ENDIAN)
+  // so that this stays an lvalue, it can't be cast to lispobj
+# define instance_layout(instance_ptr) ((uint32_t*)(instance_ptr))[1]
 #else
-  return instance_ptr[1]; // the word following the header is the layout
+# error "No instance_layout() defined"
 #endif
-}
-
-static inline struct cons *
-CONS(lispobj obj)
-{
-  return (struct cons *)(obj - LIST_POINTER_LOWTAG);
-}
-
-static inline struct vector *
-VECTOR(lispobj obj)
-{
-  return (struct vector *)(obj - OTHER_POINTER_LOWTAG);
-}
-
-static inline struct symbol *
-SYMBOL(lispobj obj)
-{
-  return (struct symbol *)(obj - OTHER_POINTER_LOWTAG);
-}
-
-static inline struct fdefn *
-FDEFN(lispobj obj)
-{
-  return (struct fdefn *)(obj - OTHER_POINTER_LOWTAG);
-}
 
 /* Is the Lisp object obj something with pointer nature (as opposed to
  * e.g. a fixnum or character or unbound marker)? */
@@ -358,7 +353,7 @@ make_lispobj(void *o, int low_tag)
 
 #define MAKE_FIXNUM(n) (n << N_FIXNUM_TAG_BITS)
 static inline lispobj
-make_fixnum(sword_t n)
+make_fixnum(uword_t n) // '<<' on negatives is _technically_ undefined behavior
 {
     return MAKE_FIXNUM(n);
 }
@@ -366,7 +361,7 @@ make_fixnum(sword_t n)
 static inline sword_t
 fixnum_value(lispobj n)
 {
-    return n >> N_FIXNUM_TAG_BITS;
+    return (sword_t)n >> N_FIXNUM_TAG_BITS;
 }
 
 static inline uword_t
@@ -375,13 +370,12 @@ code_header_words(lispobj header) // given header = code->header
   return HeaderValue(header) & SHORT_HEADER_MAX_WORDS;
 }
 
+#include "align.h"
 static inline sword_t
 code_instruction_words(lispobj n) // given n = code->code_size
 {
     /* Convert bytes into words, double-word aligned. */
-    sword_t x = ((n >> N_FIXNUM_TAG_BITS) + LOWTAG_MASK) & ~LOWTAG_MASK;
-
-    return x >> WORD_SHIFT;
+    return ALIGN_UP(fixnum_value(n), 2*N_WORD_BYTES) >> WORD_SHIFT;
 }
 #undef HEADER_VALUE_MASKED
 
@@ -456,6 +450,8 @@ extern char *copied_string (char *string);
  * the naive way: */
 #if defined(LISP_FEATURE_GENCGC) && !defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
 # define GENCGC_IS_PRECISE 1
+#else
+# define GENCGC_IS_PRECISE 0
 #endif
 
 void *os_dlsym_default(char *name);
@@ -463,6 +459,7 @@ void *os_dlsym_default(char *name);
 struct lisp_startup_options {
     boolean noinform;
 };
+extern struct lisp_startup_options lisp_startup_options;
 
 /* Even with just -O1, gcc optimizes the jumps in this "loop" away
  * entirely, giving the ability to define WITH-FOO-style macros. */
@@ -470,4 +467,26 @@ struct lisp_startup_options {
     int prefix##done = 0;                               \
     for (; !prefix##done; finally_do, prefix##done = 1)
 
+// casting to void is no longer enough to suppress a warning about unused
+// results of libc functions declared with warn_unused_result.
+// from http://git.savannah.gnu.org/cgit/gnulib.git/tree/lib/ignore-value.h
+#if 3 < __GNUC__ + (4 <= __GNUC_MINOR__)
+# define ignore_value(x) \
+      (__extension__ ({ __typeof__ (x) __x = (x); (void) __x; }))
+#else
+# define ignore_value(x) ((void) (x))
+#endif
+
+#if defined(__GNUC__) && defined(ADDRESS_SANITIZER)
+#define NO_SANITIZE_ADDRESS __attribute__((no_sanitize_address))
+#else
+#define NO_SANITIZE_ADDRESS
+#endif
+
+#if defined(__GNUC__) && defined(MEMORY_SANITIZER)
+#define NO_SANITIZE_MEMORY __attribute__((no_sanitize_memory))
+#else
+#define NO_SANITIZE_MEMORY
+#endif
+
 #endif /* _SBCL_RUNTIME_H_ */
