@@ -35,34 +35,32 @@
 #include "gc-internal.h"
 #include "thread.h"
 #include "arch.h"
-
+#include "pseudo-atomic.h"
 #include "genesis/static-symbols.h"
 #include "genesis/symbol.h"
+#include "immobile-space.h"
 
 #ifdef LISP_FEATURE_SB_CORE_COMPRESSION
 # include <zlib.h>
 #endif
 
-/* write_runtime_options uses a simple serialization scheme that
- * consists of one word of magic, one word indicating whether options
- * are actually saved, and one word per struct field. */
+#define GENERAL_WRITE_FAILURE_MSG "error writing to core file"
+
+/* write_memsize_options uses a simple serialization scheme that
+ * consists of one word of magic, one word indicating the size of the
+ * core entry, and one word per struct field. */
 static void
-write_runtime_options(FILE *file, struct runtime_options *options)
+write_memsize_options(FILE *file)
 {
-    size_t optarray[RUNTIME_OPTIONS_WORDS];
-
-    memset(&optarray, 0, sizeof(optarray));
-    optarray[0] = RUNTIME_OPTIONS_MAGIC;
-
-    if (options != NULL) {
-        /* optarray[1] is a flag indicating that options are present */
-        optarray[1] = 1;
-        optarray[2] = options->dynamic_space_size;
-        optarray[3] = options->thread_control_stack_size;
-    }
+    core_entry_elt_t optarray[RUNTIME_OPTIONS_WORDS] = {
+      RUNTIME_OPTIONS_MAGIC,
+      4, // number of words in this core header entry
+      dynamic_space_size,
+      thread_control_stack_size
+    };
 
     if (RUNTIME_OPTIONS_WORDS !=
-        fwrite(optarray, sizeof(size_t), RUNTIME_OPTIONS_WORDS, file)) {
+        fwrite(optarray, sizeof(core_entry_elt_t), RUNTIME_OPTIONS_WORDS, file)) {
         perror("Error writing runtime options to file");
     }
 }
@@ -71,7 +69,7 @@ static void
 write_lispobj(lispobj obj, FILE *file)
 {
     if (1 != fwrite(&obj, sizeof(lispobj), 1, file)) {
-        perror("Error writing to file");
+        perror(GENERAL_WRITE_FAILURE_MSG);
     }
 }
 
@@ -86,7 +84,7 @@ write_bytes_to_file(FILE * file, char *addr, long bytes, int compression)
                 addr += count;
             }
             else {
-                perror("error writing to core file");
+                perror(GENERAL_WRITE_FAILURE_MSG);
                 lose("core file is incomplete or corrupt\n");
             }
         }
@@ -94,7 +92,7 @@ write_bytes_to_file(FILE * file, char *addr, long bytes, int compression)
     } else if ((compression >= -1) && (compression <= 9)) {
 # define ZLIB_BUFFER_SIZE (1u<<16)
         z_stream stream;
-        unsigned char buf[ZLIB_BUFFER_SIZE];
+        unsigned char* buf = successful_malloc(ZLIB_BUFFER_SIZE);
         unsigned char * written, * end;
         long total_written = 0;
         int ret;
@@ -107,24 +105,25 @@ write_bytes_to_file(FILE * file, char *addr, long bytes, int compression)
         if (ret != Z_OK)
             lose("deflateInit: %i\n", ret);
         do {
-            stream.avail_out = sizeof(buf);
+            stream.avail_out = ZLIB_BUFFER_SIZE;
             stream.next_out = buf;
             ret = deflate(&stream, Z_FINISH);
             if (ret < 0) lose("zlib deflate error: %i... exiting\n", ret);
             written = buf;
-            end     = buf+sizeof(buf)-stream.avail_out;
+            end     = buf+ZLIB_BUFFER_SIZE-stream.avail_out;
             total_written += end - written;
             while (written < end) {
                 long count = fwrite(written, 1, end-written, file);
                 if (count > 0) {
                     written += count;
                 } else {
-                    perror("error writing to core file");
+                    perror(GENERAL_WRITE_FAILURE_MSG);
                     lose("core file is incomplete or corrupt\n");
                 }
             }
         } while (stream.avail_out == 0);
         deflateEnd(&stream);
+        free(buf);
         printf("compressed %lu bytes into %lu at level %i\n",
                bytes, total_written, compression);
 # undef ZLIB_BUFFER_SIZE
@@ -138,20 +137,17 @@ write_bytes_to_file(FILE * file, char *addr, long bytes, int compression)
     }
 
     if (fflush(file) != 0) {
-      perror("error writing to core file");
+      perror(GENERAL_WRITE_FAILURE_MSG);
       lose("core file is incomplete or corrupt\n");
     }
 };
 
 
-static long
-write_and_compress_bytes(FILE *file, char *addr, long bytes, os_vm_offset_t file_offset,
-                         int compression)
+static long write_bytes(FILE *file, char *addr, long bytes,
+                        os_vm_offset_t file_offset, int compression)
 {
     long here, data;
 
-    bytes = (bytes+os_vm_page_size-1)&~(os_vm_page_size-1);
-
 #ifdef LISP_FEATURE_WIN32
     long count;
     /* touch every single page in the space to force it to be mapped. */
@@ -163,27 +159,21 @@ write_and_compress_bytes(FILE *file, char *addr, long bytes, os_vm_offset_t file
     fflush(file);
     here = ftell(file);
     fseek(file, 0, SEEK_END);
-    data = (ftell(file)+os_vm_page_size-1)&~(os_vm_page_size-1);
+    data = ALIGN_UP(ftell(file), os_vm_page_size);
     fseek(file, data, SEEK_SET);
     write_bytes_to_file(file, addr, bytes, compression);
     fseek(file, here, SEEK_SET);
     return ((data - file_offset) / os_vm_page_size) - 1;
 }
 
-static long
-write_bytes(FILE *file, char *addr, long bytes, os_vm_offset_t file_offset)
-{
-    return write_and_compress_bytes(file, addr, bytes, file_offset,
-                                    COMPRESSION_LEVEL_NONE);
-}
-
 static void
 output_space(FILE *file, int id, lispobj *addr, lispobj *end,
              os_vm_offset_t file_offset,
              int core_compression_level)
 {
     size_t words, bytes, data, compressed_flag;
-    static char *names[] = {NULL, "dynamic", "static", "read-only"};
+    static char *names[] = {NULL, "dynamic", "static", "read-only",
+                            "immobile", "immobile"};
 
     compressed_flag
             = ((core_compression_level != COMPRESSION_LEVEL_NONE)
@@ -195,18 +185,19 @@ output_space(FILE *file, int id, lispobj *addr, lispobj *end,
 
     bytes = words * sizeof(lispobj);
 
-    printf("writing %lu bytes from the %s space at %p\n",
-           bytes, names[id], addr);
+    if (!lisp_startup_options.noinform)
+        printf("writing %lu bytes from the %s space at %p\n",
+               (long unsigned)bytes, names[id], addr);
 
-    data = write_and_compress_bytes(file, (char *)addr, bytes, file_offset,
-                                    core_compression_level);
+    data = write_bytes(file, (char *)addr, ALIGN_UP(bytes, os_vm_page_size),
+                       file_offset, core_compression_level);
 
     write_lispobj(data, file);
-    write_lispobj((uword_t)addr / os_vm_page_size, file);
+    write_lispobj((uword_t)addr, file);
     write_lispobj((bytes + os_vm_page_size - 1) / os_vm_page_size, file);
 }
 
-FILE *
+static FILE *
 open_core_for_saving(char *filename)
 {
     /* Open the output file. We don't actually need the file yet, but
@@ -216,94 +207,101 @@ open_core_for_saving(char *filename)
     return fopen(filename, "wb");
 }
 
+void unwind_binding_stack()
+{
+    boolean verbose = !lisp_startup_options.noinform;
+    struct thread *th = all_threads;
+
+    /* Smash the enclosing state. (Once we do this, there's no good
+     * way to go back, which is a sufficient reason that this ends up
+     * being SAVE-LISP-AND-DIE instead of SAVE-LISP-AND-GO-ON). */
+    if (verbose) {
+        printf("[undoing binding stack and other enclosing state... ");
+        fflush(stdout);
+    }
+    unbind_to_here((lispobj *)th->binding_stack_start,th);
+    write_TLS(CURRENT_CATCH_BLOCK, 0, th); // If set to 0 on start, why here too?
+    write_TLS(CURRENT_UNWIND_PROTECT_BLOCK, 0, th);
+    if (verbose) printf("done]\n");
+}
+
 boolean
 save_to_filehandle(FILE *file, char *filename, lispobj init_function,
                    boolean make_executable,
                    boolean save_runtime_options,
                    int core_compression_level)
 {
-    struct thread *th;
-    os_vm_offset_t core_start_pos;
-
-#ifdef LISP_FEATURE_X86_64
-    untune_asm_routines_for_microarch();
-#endif
-
-    /* Smash the enclosing state. (Once we do this, there's no good
-     * way to go back, which is a sufficient reason that this ends up
-     * being SAVE-LISP-AND-DIE instead of SAVE-LISP-AND-GO-ON). */
-    printf("[undoing binding stack and other enclosing state... ");
-    fflush(stdout);
-    for_each_thread(th) {       /* XXX really? */
-        unbind_to_here((lispobj *)th->binding_stack_start,th);
-        SetSymbolValue(CURRENT_CATCH_BLOCK, 0,th);
-        SetSymbolValue(CURRENT_UNWIND_PROTECT_BLOCK, 0,th);
-    }
-    printf("done]\n");
-    fflush(stdout);
+    boolean verbose = !lisp_startup_options.noinform;
 
     /* (Now we can actually start copying ourselves into the output file.) */
 
-    printf("[saving current Lisp image into %s:\n", filename);
-    fflush(stdout);
-
-    core_start_pos = ftell(file);
-    write_lispobj(CORE_MAGIC, file);
-
-    write_lispobj(BUILD_ID_CORE_ENTRY_TYPE_CODE, file);
-    write_lispobj(/* (We're writing the word count of the entry here, and the 2
-          * term is one word for the leading BUILD_ID_CORE_ENTRY_TYPE_CODE
-          * word and one word where we store the count itself.) */
-         2 + strlen((const char *)build_id),
-         file);
-    {
-        unsigned char *p;
-        for (p = (unsigned char *)build_id; *p; ++p)
-            write_lispobj(*p, file);
+    if (verbose) {
+        printf("[saving current Lisp image into %s:\n", filename);
+        fflush(stdout);
     }
 
-    write_lispobj(NEW_DIRECTORY_CORE_ENTRY_TYPE_CODE, file);
-    write_lispobj(/* (word count = 3 spaces described by 5 words each, plus the
+    os_vm_offset_t core_start_pos = ftell(file);
+    write_lispobj(CORE_MAGIC, file);
+
+    /* If 'save_runtime_options' is specified then the saved thread stack size
+     * and dynamic space size are used in the restarted image and
+     * all command-line arguments are available to Lisp in SB-EXT:*POSIX-ARGV*.
+     * Otherwise command-line processing is performed as normal */
+    if (save_runtime_options)
+        write_memsize_options(file);
+
+    int stringlen = strlen((const char *)build_id);
+    int string_words = ALIGN_UP(stringlen, sizeof (core_entry_elt_t))
+        / sizeof (core_entry_elt_t);
+    int pad = string_words * sizeof (core_entry_elt_t) - stringlen;
+    /* Write 3 word entry header: a word for entry-type-code, a word for
+     * the total length in words, and a word for the string length */
+    write_lispobj(BUILD_ID_CORE_ENTRY_TYPE_CODE, file);
+    write_lispobj(3 + string_words, file);
+    write_lispobj(stringlen, file);
+    int nwrote = fwrite(build_id, 1, stringlen, file);
+    /* Write padding bytes to align to core_entry_elt_t */
+    while (pad--) nwrote += (fputc(0xff, file) != EOF);
+    if (nwrote != (int)(sizeof (core_entry_elt_t) * string_words))
+        perror(GENERAL_WRITE_FAILURE_MSG);
+
+    write_lispobj(DIRECTORY_CORE_ENTRY_TYPE_CODE, file);
+    write_lispobj(/* (word count = N spaces described by 5 words each, plus the
           * entry type code, plus this count itself) */
-         (5*3)+2, file);
+         (5 * MAX_CORE_SPACE_ID) + 2, file);
     output_space(file,
                  READ_ONLY_CORE_SPACE_ID,
                  (lispobj *)READ_ONLY_SPACE_START,
-                 (lispobj *)SymbolValue(READ_ONLY_SPACE_FREE_POINTER,0),
+                 read_only_space_free_pointer,
                  core_start_pos,
                  core_compression_level);
     output_space(file,
                  STATIC_CORE_SPACE_ID,
                  (lispobj *)STATIC_SPACE_START,
-                 (lispobj *)SymbolValue(STATIC_SPACE_FREE_POINTER,0),
+                 static_space_free_pointer,
                  core_start_pos,
                  core_compression_level);
-#ifdef LISP_FEATURE_GENCGC
-    /* Flush the current_region, updating the tables. */
-    gc_alloc_update_all_page_tables();
-    update_dynamic_space_free_pointer();
-#endif
-#ifdef reg_ALLOC
-#ifdef LISP_FEATURE_GENCGC
     output_space(file,
                  DYNAMIC_CORE_SPACE_ID,
-                 (lispobj *)DYNAMIC_SPACE_START,
-                 dynamic_space_free_pointer,
+                 current_dynamic_space,
+                 (lispobj *)get_alloc_pointer(),
                  core_start_pos,
                  core_compression_level);
-#else
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
     output_space(file,
-                 DYNAMIC_CORE_SPACE_ID,
-                 (lispobj *)current_dynamic_space,
-                 dynamic_space_free_pointer,
+                 IMMOBILE_FIXEDOBJ_CORE_SPACE_ID,
+                 (lispobj *)FIXEDOBJ_SPACE_START,
+                 fixedobj_free_pointer,
                  core_start_pos,
                  core_compression_level);
-#endif
-#else
+    // Leave this space for last! Things are easier when splitting a core into
+    // code and non-code if we don't have to compensate for removal of pages.
+    // i.e. if code resided between dynamic and fixedobj space, then dynamic
+    // space would need to have it pages renumbered when code is pulled out.
     output_space(file,
-                 DYNAMIC_CORE_SPACE_ID,
-                 (lispobj *)DYNAMIC_SPACE_START,
-                 (lispobj *)SymbolValue(ALLOCATION_POINTER,0),
+                 IMMOBILE_VARYOBJ_CORE_SPACE_ID,
+                 (lispobj *)VARYOBJ_SPACE_START,
+                 varyobj_free_pointer,
                  core_start_pos,
                  core_compression_level);
 #endif
@@ -314,28 +312,21 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
 
 #ifdef LISP_FEATURE_GENCGC
     {
-        size_t size = (last_free_page*sizeof(sword_t)+os_vm_page_size-1)
-            &~(os_vm_page_size-1);
-        uword_t *data = calloc(size, 1);
-        if (data) {
-            uword_t word;
-            sword_t offset;
-            page_index_t i;
-            for (i = 0; i < last_free_page; i++) {
-                /* Thanks to alignment requirements, the two low bits
-                 * are always zero, so we can use them to store the
-                 * allocation type -- region is always closed, so only
-                 * the two low bits of allocation flags matter. */
-                word = page_table[i].scan_start_offset;
-                gc_assert((word & 0x03) == 0);
-                data[i] = word | (0x03 & page_table[i].allocated);
-            }
-            write_lispobj(PAGE_TABLE_CORE_ENTRY_TYPE_CODE, file);
-            write_lispobj(4, file);
-            write_lispobj(size, file);
-            offset = write_bytes(file, (char *)data, size, core_start_pos);
-            write_lispobj(offset, file);
-        }
+        extern void gc_store_corefile_ptes(struct corefile_pte*);
+        size_t true_size = next_free_page * sizeof(struct corefile_pte);
+        size_t aligned_size = ALIGN_UP(true_size, N_WORD_BYTES);
+        char* data = successful_malloc(aligned_size);
+        // Zeroize the final few bytes of data that get written out
+        // but might be untouched by gc_store_corefile_ptes().
+        memset(data + aligned_size - N_WORD_BYTES, 0, N_WORD_BYTES);
+        gc_store_corefile_ptes((struct corefile_pte*)data);
+        write_lispobj(PAGE_TABLE_CORE_ENTRY_TYPE_CODE, file);
+        write_lispobj(5, file); // 5 = # of words in this core header entry
+        write_lispobj(next_free_page, file);
+        write_lispobj(aligned_size, file);
+        sword_t offset = write_bytes(file, data, aligned_size, core_start_pos,
+                                     COMPRESSION_LEVEL_NONE);
+        write_lispobj(offset, file);
     }
 #endif
 
@@ -346,14 +337,6 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
      * prepended to it. */
     fseek(file, 0, SEEK_END);
 
-    /* If NULL runtime options are passed to write_runtime_options,
-     * command-line processing is performed as normal in the SBCL
-     * executable. Otherwise, the saved runtime options are used and
-     * all command-line arguments are available to Lisp in
-     * SB-EXT:*POSIX-ARGV*. */
-    write_runtime_options(file,
-                          (save_runtime_options ? runtime_options : NULL));
-
     if (1 != fwrite(&core_start_pos, sizeof(os_vm_offset_t), 1, file)) {
         perror("Error writing core starting position to file");
         fclose(file);
@@ -367,13 +350,13 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
         chmod (filename, 0755);
 #endif
 
-    printf("done]\n");
+    if (verbose) printf("done]\n");
     exit(0);
 }
 
 /* Check if the build_id for the current runtime is present in a
  * buffer. */
-int
+static int
 check_runtime_build_id(void *buf, size_t size)
 {
     size_t idlen;
@@ -394,7 +377,7 @@ check_runtime_build_id(void *buf, size_t size)
  * and return it.  Places the size in bytes of the runtime into
  * 'size_out'.  Returns NULL if the runtime cannot be loaded from
  * 'runtime_path'. */
-void *
+static void *
 load_runtime(char *runtime_path, size_t *size_out)
 {
     void *buf = NULL;
@@ -402,7 +385,7 @@ load_runtime(char *runtime_path, size_t *size_out)
     size_t size, count;
     os_vm_offset_t core_offset;
 
-    core_offset = search_for_embedded_core (runtime_path);
+    core_offset = search_for_embedded_core (runtime_path, 0);
     if ((input = fopen(runtime_path, "rb")) == NULL) {
         fprintf(stderr, "Unable to open runtime: %s\n", runtime_path);
         goto lose;
@@ -495,6 +478,14 @@ prepare_to_save(char *filename, boolean prepend_runtime, void **runtime_bytes,
 {
     FILE *file;
     char *runtime_path;
+    extern char *saved_runtime_path; // path computed from argv[0]
+
+    // SB-IMPL::DEINIT already checked for exactly 1 thread,
+    // so this really shouldn't happen.
+    if (all_threads->next) {
+        fprintf(stderr, "Can't save image with more than one executing thread");
+        return NULL;
+    }
 
     if (prepend_runtime) {
         runtime_path = os_get_runtime_executable_path(0);
@@ -525,6 +516,7 @@ prepare_to_save(char *filename, boolean prepend_runtime, void **runtime_bytes,
     return file;
 }
 
+#ifdef LISP_FEATURE_CHENEYGC
 boolean
 save(char *filename, lispobj init_function, boolean prepend_runtime,
      boolean save_runtime_options, boolean compressed, int compression_level,
@@ -541,7 +533,13 @@ save(char *filename, lispobj init_function, boolean prepend_runtime,
     if (prepend_runtime)
         save_runtime_to_filehandle(file, runtime_bytes, runtime_size, application_type);
 
+    /* This unwinding is necessary for proper restoration of the
+     * symbol-value slots to their toplevel values, but it occurs
+     * too late to remove old references from the binding stack.
+     * There's probably no safe way to do that from Lisp */
+    unwind_binding_stack();
     return save_to_filehandle(file, filename, init_function, prepend_runtime,
                               save_runtime_options,
                               compressed ? compressed : COMPRESSION_LEVEL_NONE);
 }
+#endif
