@@ -4,13 +4,18 @@
 #include "runtime.h"
 #include "lispregs.h"
 #include "gc-internal.h"
+#include "gc-private.h"
 #include "genesis/closure.h"
 #include "genesis/cons.h"
 #include "genesis/constants.h"
 #include "genesis/gc-tables.h"
+#include "genesis/hash-table.h"
+#include "genesis/instance.h"
 #include "genesis/layout.h"
 #include "genesis/package.h"
+#include "genesis/vector.h"
 #include "pseudo-atomic.h" // for get_alloc_pointer()
+#include "search.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -68,10 +73,9 @@ static int gen_of(lispobj obj) {
     return -1;
 }
 
-const char* classify_obj(lispobj ptr)
+static const char* classify_obj(lispobj ptr)
 {
     extern lispobj* instance_classoid_name(lispobj*);
-    extern char *widetag_names[];
 
     lispobj* name; // a Lisp string
     switch(lowtag_of(ptr)) {
@@ -94,10 +98,7 @@ static void add_to_layer(lispobj* obj, int wordindex,
                          struct layer* layer, int* capacity)
 {
     // Resurrect the containing object's lowtag
-    lispobj word = *obj;
-    int lowtag = is_cons_half(word) ?
-      LIST_POINTER_LOWTAG : lowtag_for_widetag[widetag_of(word)>>2];
-    lispobj ptr = make_lispobj(obj, lowtag);
+    lispobj ptr = compute_lispobj(obj);
     int staticp = ptr <= STATIC_SPACE_END;
     int gen = staticp ? -1 : gen_of(ptr);
     if (heap_trace_verbose>2)
@@ -120,8 +121,7 @@ static void add_to_layer(lispobj* obj, int wordindex,
 /// otherwise return obj directly.
 static lispobj canonical_obj(lispobj obj)
 {
-    if (lowtag_of(obj) == FUN_POINTER_LOWTAG &&
-        widetag_of(*native_pointer(obj)) == SIMPLE_FUN_WIDETAG)
+    if (functionp(obj) && widetag_of(*native_pointer(obj)) == SIMPLE_FUN_WIDETAG)
         return make_lispobj(fun_code_header(obj-FUN_POINTER_LOWTAG),
                             OTHER_POINTER_LOWTAG);
     return obj;
@@ -155,13 +155,15 @@ static int find_ref(lispobj* source, lispobj target)
         // here we just go for clarity by abstracting out logbitp.
         layout = instance_layout(source);
         check_ptr(0, layout);
-        bitmap = ((struct layout*)native_pointer(layout))->bitmap;
+        bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
         for(i=1; i<scan_limit; ++i)
             if (layout_bitmap_logbitp(i-1, bitmap)) check_ptr(i, source[i]);
         return -1;
+#if FUN_SELF_FIXNUM_TAGGED
     case CLOSURE_WIDETAG:
         check_ptr(1, ((struct closure*)source)->fun - FUN_RAW_ADDR_OFFSET);
         break;
+#endif
     case CODE_HEADER_WIDETAG:
         for_each_simple_fun(i, function_ptr, (struct code*)source, 0, {
             int wordindex = &function_ptr->name - source;
@@ -169,12 +171,10 @@ static int find_ref(lispobj* source, lispobj target)
         })
         scan_limit = code_header_words(header);
         break;
-#ifdef LISP_FEATURE_IMMOBILE_CODE
     case FDEFN_WIDETAG:
-        check_ptr(3, fdefn_raw_referent((struct fdefn*)source));
+        check_ptr(3, fdefn_callee_lispobj((struct fdefn*)source));
         scan_limit = 3;
         break;
-#endif
     }
     for(i=1; i<scan_limit; ++i) check_ptr(i, source[i]);
     return -1;
@@ -184,54 +184,24 @@ static int find_ref(lispobj* source, lispobj target)
 enum ref_kind { HEAP, CONTROL_STACK, BINDING_STACK, TLS };
 char *ref_kind_name[4] = {"heap","C stack","bindings","TLS"};
 
-/// This unfortunately entails a heap scan,
-/// but it's quite fast if the symbol is found in immobile space.
-static lispobj* find_sym_by_tls_index(unsigned int tls_index)
-{
-    lispobj* where = 0;
-    lispobj* end = 0;
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-    where = (lispobj*)IMMOBILE_SPACE_START;
-    end = (lispobj*)SYMBOL(IMMOBILE_FIXEDOBJ_FREE_POINTER)->value;
-#endif
-    while (1) {
-        while (where < end) {
-            lispobj header = *where;
-            int widetag = widetag_of(header);
-            if (widetag == SYMBOL_WIDETAG &&
-                tls_index_of(((struct symbol*)where)) == tls_index)
-                return where;
-            where += is_cons_half(header) ? 2 : sizetab[widetag](where);
-        }
-        if (where >= (lispobj*)DYNAMIC_SPACE_START)
-            break;
-        where = (lispobj*)DYNAMIC_SPACE_START;
-        end = (lispobj*)get_alloc_pointer();
-    }
-    return 0;
-}
-
 static inline int interestingp(lispobj ptr, struct hopscotch_table* targets)
 {
     return is_lisp_pointer(ptr) && hopscotch_containsp(targets, ptr);
 }
 
+#ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
 /* Try to find the call frame that contains 'addr', which is the address
  * in which a conservative root was seen.
  * Return the program counter associated with that frame. */
-static char* deduce_thread_pc(struct thread* th, void** addr)
+static char* NO_SANITIZE_MEMORY deduce_thread_pc(struct thread* th, void** addr)
 {
     uword_t* fp = __builtin_frame_address(0);
     char* return_pc = 0;
 
     if (th != arch_os_get_current_thread()) {
-        int i = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,th));
-        os_context_t *c = th->interrupt_contexts[i-1];
-#ifdef LISP_FEATURE_64_BIT
-        fp = (uword_t*)*os_context_register_addr(c,reg_RBP);
-#else
-        fp = (uword_t*)*os_context_register_addr(c,reg_EBP);
-#endif
+        int i = fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,th));
+        os_context_t *c = nth_interrupt_context(i-1, th);
+        fp = (uword_t*)*os_context_register_addr(c,reg_FP);
     }
     while (1) {
         if ((uword_t*)addr < fp)
@@ -252,8 +222,8 @@ static void compare_pointer(void *addr) {
 
 /* Figure out which thread's control stack contains 'pointer'
  * and the PC within the active function in the referencing frame  */
-static struct thread* deduce_thread(void (*context_scanner)(),
-                                    uword_t pointer, char** pc)
+static struct thread* NO_SANITIZE_MEMORY
+deduce_thread(void (*context_scanner)(), uword_t pointer, char** pc)
 {
     struct thread *th;
 
@@ -266,9 +236,9 @@ static struct thread* deduce_thread(void (*context_scanner)(),
             esp = (void **)((void *)&pointer);
         else {
             void **esp1;
-            free = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,th));
+            free = fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,th));
             for(i=free-1;i>=0;i--) {
-                os_context_t *c=th->interrupt_contexts[i];
+                os_context_t *c = nth_interrupt_context(i, th);
                 esp1 = (void **) *os_context_register_addr(c,reg_SP);
                 if (esp1>=(void **)th->control_stack_start && esp1<(void **)th->control_stack_end) {
                     if(esp1<esp) esp=esp1;
@@ -290,16 +260,18 @@ static struct thread* deduce_thread(void (*context_scanner)(),
     }
     return 0;
 }
+#endif
 
+/* KNOWN BUG: stack reference to pinned large object or immobile object
+ * won't be found in pins hashtable */
 static lispobj examine_stacks(struct hopscotch_table* targets,
                               void (*context_scanner)(),
                               int n_pins, lispobj* pins,
                               enum ref_kind *root_kind,
                               struct thread** root_thread,
                               char** thread_pc,
-                              unsigned int *tls_index)
+                              lispobj *tls_index)
 {
-    boolean world_stopped = context_scanner != 0;
     struct thread *th;
 
     for_each_thread(th) {
@@ -326,7 +298,31 @@ static lispobj examine_stacks(struct hopscotch_table* targets,
                 *tls_index = where[1];
                 return *where;
             }
+#ifndef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
+        *root_kind = CONTROL_STACK;
+        // Examine the control stack
+        where = th->control_stack_start;
+        end = access_control_stack_pointer(th);
+        for ( ; where < end ; ++where)
+            if (interestingp(*where, targets)) {
+                *root_thread = th;
+                *thread_pc = 0;
+                return *where;
+            }
+        // Examine the explicit pin list
+        lispobj pin_list = read_TLS(PINNED_OBJECTS,th);
+        while (pin_list != NIL) {
+            uword_t pin = CONS(pin_list)->car;
+            if (interestingp(pin, targets)) {
+                *root_thread = th;
+                *thread_pc = 0;
+                return *where;
+            }
+            pin_list = CONS(pin_list)->cdr;
+        }
+#endif
     }
+#ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
     // Look in the control stacks
     *root_kind = CONTROL_STACK;
     uword_t pin;
@@ -334,6 +330,7 @@ static lispobj examine_stacks(struct hopscotch_table* targets,
     for (i=n_pins-1; i>=0; --i)
         // Bypass interestingp() to avoid one test - pins are known pointers.
         if (hopscotch_containsp(targets, pin = pins[i])) {
+            boolean world_stopped = context_scanner != 0;
             if (world_stopped) {
                 *root_thread = deduce_thread(context_scanner, pin, thread_pc);
             } else {
@@ -345,19 +342,20 @@ static lispobj examine_stacks(struct hopscotch_table* targets,
                 void **esp = __builtin_frame_address(0);
                 void **where;
                 for (where = ((void **)th->control_stack_end)-1; where >= esp;  --where)
-                  if (*where == (void*)pin) {
-                      *root_thread = th;
-                      *thread_pc = deduce_thread_pc(th, where);
-                      break;
-                  }
+                    if (*where == (void*)pin) {
+                        *root_thread = th;
+                        *thread_pc = deduce_thread_pc(th, where);
+                        break;
+                    }
             }
             return pin;
         }
+#endif
     *root_kind = HEAP;
     return 0;
 }
 
-void free_graph(struct layer* layer)
+static void free_graph(struct layer* layer)
 {
     while (layer) {
         free(layer->nodes);
@@ -367,7 +365,7 @@ void free_graph(struct layer* layer)
     }
 }
 
-struct node* find_node(struct layer* layer, lispobj ptr)
+static struct node* find_node(struct layer* layer, lispobj ptr)
 {
     int i;
     for(i=layer->count-1; i>=0; --i)
@@ -402,7 +400,7 @@ static inline lispobj decode_pointer(uint32_t encoding)
         return encoding; // Literal pointer
 }
 
-struct simple_fun* simple_fun_from_pc(char* pc)
+static struct simple_fun* simple_fun_from_pc(char* pc)
 {
     struct code* code = (struct code*)component_ptr_from_pc((lispobj*)pc);
     if (!code) return 0;
@@ -423,14 +421,17 @@ static void maybe_show_object_name(lispobj obj, FILE* stream)
     if (lowtag_of(obj)==OTHER_POINTER_LOWTAG)
         switch(widetag_of(*native_pointer(obj))) {
         case SYMBOL_WIDETAG:
-            package = SYMBOL(obj)->package;
-            package_name = ((struct package*)native_pointer(package))->_name;
             putc(',', stream);
-            safely_show_lstring(native_pointer(package_name), 0, stream);
-            fputs("::", stream);
+            if ((package = SYMBOL(obj)->package) == NIL) {
+                fprintf(stream, "#:");
+            } else {
+                package_name = ((struct package*)native_pointer(package))->_name;
+                safely_show_lstring(native_pointer(package_name), 0, stream);
+                fputs("::", stream);
+            }
             safely_show_lstring(native_pointer(SYMBOL(obj)->name), 0, stream);
             break;
-      }
+        }
 }
 
 static boolean root_p(lispobj ptr, int criterion)
@@ -458,7 +459,7 @@ static void trace1(lispobj object,
     enum ref_kind root_kind;
     struct thread* root_thread;
     char* thread_pc = 0;
-    unsigned int tls_index;
+    lispobj tls_index;
     lispobj target;
     int i;
 
@@ -490,7 +491,7 @@ static void trace1(lispobj object,
                         fprintf(stderr, "%p ", ptr);
                     else {
                         lispobj word = *ptr;
-                        int nwords = is_cons_half(word) ? 2 : sizetab[widetag_of(word)](ptr);
+                        int nwords = OBJECT_SIZE(word, ptr);
                         fprintf(stderr, "%p+%d ", ptr, nwords);
                     }
                     list1 = cell[1];
@@ -570,11 +571,16 @@ static void trace1(lispobj object,
             fprintf(file, "thread=%p", root_thread);
         fprintf(file, ":%s:", ref_kind_name[root_kind]);
         if (root_kind==BINDING_STACK || root_kind==TLS) {
-            lispobj* symbol = find_sym_by_tls_index(tls_index);
+#ifdef LISP_FEATURE_SB_THREAD
+            lispobj* symbol = (lispobj*)
+              lisp_symbol_from_tls_index(tls_index);
+#else
+            lispobj* symbol = native_pointer(tls_index);
+#endif
             if (symbol)
                 show_lstring(symbol_name(symbol), 0, file);
             else
-                fprintf(file, "%x", tls_index);
+                fprintf(file, "%"OBJ_FMTX, tls_index);
         } else {
             struct simple_fun* fun = simple_fun_from_pc(thread_pc);
             if (fun) {
@@ -593,7 +599,7 @@ static void trace1(lispobj object,
                 native_pointer(anchor->object), anchor->wordindex);
     }
 
-    target = 0;
+    target = thread_ref;
     while (top_layer) {
         struct node next = *anchor;
         lispobj ptr = next.object;
@@ -603,27 +609,25 @@ static void trace1(lispobj object,
             fprintf(file, "(g%d,", gen_of(ptr));
         fputs(classify_obj(ptr), file);
         maybe_show_object_name(ptr, file);
-        fprintf(file, ")%p[%d]->", (void*)ptr, next.wordindex);
+        fprintf(file, ")#x%"OBJ_FMTX"[%d]->", ptr, next.wordindex);
         target = native_pointer(ptr)[next.wordindex];
         // Special-case a few combinations of <type,wordindex>
         switch (next.wordindex) {
         case 0:
-            if (lowtag_of(ptr) == INSTANCE_POINTER_LOWTAG  ||
-                lowtag_of(ptr) == FUN_POINTER_LOWTAG)
+            if (instancep(ptr) || functionp(ptr))
                 target = instance_layout(native_pointer(ptr));
             break;
+#if FUN_SELF_FIXNUM_TAGGED
         case 1:
-            if (lowtag_of(ptr) == FUN_POINTER_LOWTAG &&
-                widetag_of(*native_pointer(ptr)) == CLOSURE_WIDETAG)
+            if (functionp(ptr) && widetag_of(*native_pointer(ptr)) == CLOSURE_WIDETAG)
                 target -= FUN_RAW_ADDR_OFFSET;
             break;
-#ifdef LISP_FEATURE_IMMOBILE_CODE
+#endif
         case 3:
             if (lowtag_of(ptr) == OTHER_POINTER_LOWTAG &&
                 widetag_of(FDEFN(ptr)->header) == FDEFN_WIDETAG)
-                target = fdefn_raw_referent((struct fdefn*)native_pointer(ptr));
+                target = fdefn_callee_lispobj((struct fdefn*)native_pointer(ptr));
             break;
-#endif
         }
         target = canonical_obj(target);
         struct layer* next_layer = top_layer->next;
@@ -637,7 +641,8 @@ static void trace1(lispobj object,
             gc_assert(object == target);
         }
     }
-    fprintf(file, "%p.\n", (void*)target);
+    fprintf(file, "#x%"OBJ_FMTX".\n", target);
+    fflush(file);
 }
 
 static void record_ptr(lispobj* source, lispobj target,
@@ -658,16 +663,12 @@ static void record_ptr(lispobj* source, lispobj target,
                   (sword_t)((char*)new_cell - ss->scratchpad.base));
 }
 
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-#define relevant_ptr_p(x) find_page_index(x)>=0||find_immobile_page_index(x)>=0
-#else
-#define relevant_ptr_p(x) find_page_index(x)>=0
-#endif
+#define relevant_ptr_p(x) find_page_index(x)>=0||immobile_space_p((lispobj)x)
 
 #define check_ptr(ptr) { \
     ++n_scanned_words; \
     if (!is_lisp_pointer(ptr)) ++n_immediates; \
-    else if (relevant_ptr_p((void*)ptr)) { \
+    else if (relevant_ptr_p((void*)(ptr))) { \
       ++n_pointers; \
       if (record_ptrs) record_ptr(where,ptr,ss); \
     }}
@@ -675,7 +676,7 @@ static void record_ptr(lispobj* source, lispobj target,
 static uword_t build_refs(lispobj* where, lispobj* end,
                           struct scan_state* ss)
 {
-    lispobj layout, bitmap, fun;
+    lispobj layout, bitmap;
     sword_t nwords, scan_limit, i, j;
     uword_t n_objects = 0, n_scanned_words = 0,
             n_immediates = 0, n_pointers = 0;
@@ -700,16 +701,18 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             // mixed boxed/unboxed objects
             layout = instance_layout(where);
             check_ptr(layout);
-            bitmap = ((struct layout*)native_pointer(layout))->bitmap;
+            // Partially initialized instance can't have nonzero words yet
+            bitmap = layout ? LAYOUT(layout)->bitmap : make_fixnum(-1);
             // If no raw slots, just scan without use of the bitmap.
             if (bitmap == make_fixnum(-1)) break;
             for(i=1; i<scan_limit; ++i)
                 if (layout_bitmap_logbitp(i-1, bitmap)) check_ptr(where[i]);
             continue;
+#if FUN_SELF_FIXNUM_TAGGED
         case CLOSURE_WIDETAG:
-            fun = ((struct closure*)where)->fun - FUN_RAW_ADDR_OFFSET;
-            check_ptr(fun);
+            check_ptr(((struct closure*)where)->fun - FUN_RAW_ADDR_OFFSET);
             break;
+#endif
         case CODE_HEADER_WIDETAG:
             for_each_simple_fun(i, function_ptr, (struct code*)where, 0, {
                 int wordindex = &function_ptr->name - where;
@@ -717,12 +720,31 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             })
             scan_limit = code_header_words(header);
             break;
-#ifdef LISP_FEATURE_IMMOBILE_CODE
         case FDEFN_WIDETAG:
-            check_ptr(fdefn_raw_referent((struct fdefn*)where));
+            check_ptr(fdefn_callee_lispobj((struct fdefn*)where));
             scan_limit = 3;
             break;
-#endif
+        case SIMPLE_VECTOR_WIDETAG:
+            // For weak hash-table vectors, a <k,v> pair may participate in
+            // a path from the root only if it does not involve whichever
+            // object is definitely weak. This fails on weak key-OR-value
+            // tables since we can't decide whether to allow the entry.
+            if (is_vector_subtype(*where, VectorValidHashing)) {
+                lispobj lhash_table = where[2];
+                gc_assert(instancep(lhash_table));
+                struct hash_table* hash_table =
+                  (struct hash_table *)native_pointer(lhash_table);
+                int weakness = hashtable_weakness(hash_table);
+                if (hashtable_weakp(hash_table) &&
+                    (weakness == 1 || weakness == 2)) { // 1=key, 2=value
+                    // Skip the first 4 words which are:
+                    //  header, length, vector->table backpointer, rehash-p
+                    int start = (weakness == 1) ? 5 : 4;
+                    for(i=start; i<scan_limit; i+=2) check_ptr(where[i]);
+                    continue;
+                }
+            }
+            break;
         default:
             if (!(other_immediate_lowtag_p(widetag) && lowtag_for_widetag[widetag>>2]))
               lose("Unknown widetag %x\n", widetag);
@@ -748,16 +770,10 @@ static uword_t build_refs(lispobj* where, lispobj* end,
 
 static void scan_spaces(struct scan_state* ss)
 {
-    build_refs((lispobj*)STATIC_SPACE_START,
-               (lispobj*)SYMBOL(STATIC_SPACE_FREE_POINTER)->value,
-               ss);
+    build_refs((lispobj*)STATIC_SPACE_START, static_space_free_pointer, ss);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-    build_refs((lispobj*)IMMOBILE_SPACE_START,
-               (lispobj*)SYMBOL(IMMOBILE_FIXEDOBJ_FREE_POINTER)->value,
-               ss);
-    build_refs((lispobj*)IMMOBILE_VARYOBJ_SUBSPACE_START,
-               (lispobj*)SYMBOL(IMMOBILE_SPACE_FREE_POINTER)->value,
-               ss);
+    build_refs((lispobj*)FIXEDOBJ_SPACE_START, fixedobj_free_pointer, ss);
+    build_refs((lispobj*)VARYOBJ_SPACE_START, varyobj_free_pointer, ss);
 #endif
     walk_generation((uword_t(*)(lispobj*,lispobj*,uword_t))build_refs,
                     -1, (uword_t)ss);
@@ -786,8 +802,8 @@ static void compute_heap_inverse(struct hopscotch_table* inverted_heap,
     // two words of the scratchpad.
     uword_t scratchpad_min_size = (1 + ss.n_pointers) * 2 * sizeof (uint32_t);
     int pagesize = getpagesize();
-    uword_t scratchpad_size = CEILING(scratchpad_min_size, pagesize);
-    ss.scratchpad.base = os_validate(0, scratchpad_size);
+    uword_t scratchpad_size = ALIGN_UP(scratchpad_min_size, pagesize);
+    ss.scratchpad.base = os_allocate(scratchpad_size);
     gc_assert(ss.scratchpad.base);
     ss.scratchpad.free = ss.scratchpad.base + 2 * sizeof(uint32_t);
     ss.scratchpad.end  = ss.scratchpad.base + scratchpad_size;
@@ -854,9 +870,9 @@ static void trace_paths(void (*context_scanner)(),
         }
     } while (weak_pointers != NIL);
     os_invalidate(scratchpad.base, scratchpad.end-scratchpad.base);
-    hopscotch_delete(&inverted_heap);
-    hopscotch_delete(&visited);
-    hopscotch_delete(&targets);
+    hopscotch_destroy(&inverted_heap);
+    hopscotch_destroy(&visited);
+    hopscotch_destroy(&targets);
 }
 
 void gc_prove_liveness(void(*context_scanner)(),
@@ -866,9 +882,7 @@ void gc_prove_liveness(void(*context_scanner)(),
 {
     int n_watched = 0, n_live = 0, n_bad = 0;
     lispobj list;
-    for ( list = objects ;
-          list != NIL && lowtag_of(list) == LIST_POINTER_LOWTAG ;
-          list = CONS(list)->cdr ) {
+    for (list = objects ; list != NIL && listp(list) ; list = CONS(list)->cdr) {
         ++n_watched;
         lispobj car = CONS(list)->car;
         if ((lowtag_of(car) != OTHER_POINTER_LOWTAG ||
@@ -878,7 +892,7 @@ void gc_prove_liveness(void(*context_scanner)(),
             n_live += ((struct weak_pointer*)native_pointer(car))->value
                 != UNBOUND_MARKER_WIDETAG;
     }
-    if (lowtag_of(list) != LIST_POINTER_LOWTAG || n_bad) {
+    if (!listp(list) || n_bad) {
         fprintf(stderr, "; Bad value in liveness tracker\n");
         return;
     }
@@ -892,11 +906,7 @@ void gc_prove_liveness(void(*context_scanner)(),
     if (n_pins>0 && !is_lisp_pointer(pins[0])) {
         int i;
         for(i=n_pins-1; i>=0; --i) {
-            lispobj* obj = (lispobj*)pins[i];
-            lispobj header = *obj;
-            int lowtag = is_cons_half(header) ? LIST_POINTER_LOWTAG
-                : lowtag_for_widetag[widetag_of(header)>>2];
-            pins[i] = make_lispobj(obj, lowtag);
+            pins[i] = compute_lispobj((lispobj*)pins[i]);
         }
     }
     trace_paths(context_scanner, objects, n_pins, (lispobj*)pins, criterion);
@@ -912,48 +922,6 @@ void prove_liveness(lispobj objects, int criterion)
     gc_prove_liveness(0, objects, gc_n_stack_pins, pinned_objects.keys, criterion);
 }
 
-#include "genesis/package.h"
-#include "genesis/instance.h"
-#include "genesis/vector.h"
-
-static boolean __attribute__((unused)) sym_stringeq(lispobj sym, const char *string, int len)
-{
-    struct vector* name = (struct vector*)native_pointer(SYMBOL(sym)->name);
-    return widetag_of(name->header) == SIMPLE_BASE_STRING_WIDETAG
-        && fixnum_value(name->length) == len
-        && !strcmp((char*)name->data, string);
-}
-
-/* Return the value of SB-THREAD::*ALL-THREADS*
- * This does not need to be particularly efficient.
- */
-static const char __attribute__((unused)) all_threads_sym[] = "*ALL-THREADS*";
-static lispobj all_lisp_threads()
-{
-#ifdef ENTER_FOREIGN_CALLBACK
-    // Starting with a known static symbol in SB-THREAD::, get the SB-THREAD package
-    // and find *ALL-THREADS* (which isn't static). Fewer static symbols is better.
-    struct symbol*   sym = SYMBOL(ENTER_FOREIGN_CALLBACK);
-    struct package*  pkg = (struct package*)native_pointer(sym->package);
-    struct instance* internals = (struct instance*)native_pointer(pkg->internal_symbols);
-    struct vector*   cells = (struct vector*)
-        native_pointer(internals->slots[INSTANCE_DATA_START]);
-    int cells_length = fixnum_value(cells->length);
-    static int index = 0;
-    int initial_index = index;
-    do {
-        lispobj thing = cells->data[index];
-        if (lowtag_of(thing) == OTHER_POINTER_LOWTAG
-            && widetag_of(SYMBOL(thing)->header) == SYMBOL_WIDETAG
-            && sym_stringeq(thing, all_threads_sym, strlen(all_threads_sym)))
-          return SYMBOL(thing)->value;
-        index = (index + 1) % cells_length;
-    } while (index != initial_index);
-    lose("Can't find *ALL-THREADS*");
-#endif
-    return NIL;
-}
-
 // These are slot offsets in (DEFSTRUCT THREAD),
 // not the C structure defined in genesis/thread.h
 #define LISP_THREAD_NAME_SLOT INSTANCE_DATA_START+0
@@ -961,7 +929,9 @@ static lispobj all_lisp_threads()
 
 struct vector* lisp_thread_name(os_thread_t os_thread)
 {
-    lispobj list = all_lisp_threads();
+    static unsigned int hint;
+    lispobj* sym = find_symbol("*ALL-THREADS*", "SB-THREAD", &hint);
+    lispobj list = sym ? ((struct symbol*)sym)->value : NIL;
     while (list != NIL) {
         struct instance* lisp_thread = (struct instance*)native_pointer(CONS(list)->car);
         list = CONS(list)->cdr;
