@@ -58,7 +58,7 @@
 
 #include "validate.h"
 #include "thread.h"
-#include "cpputil.h"
+#include "align.h"
 
 #ifndef LISP_FEATURE_SB_THREAD
 /* dummy definition to reduce ifdef clutter */
@@ -235,14 +235,6 @@ unsigned long block_deferrables_and_return_mask()
     return (unsigned long)sset;
 }
 
-#if defined(LISP_FEATURE_SB_THREAD)
-void apply_sigmask(unsigned long sigmask)
-{
-    sigset_t sset = (sigset_t)sigmask;
-    thread_sigmask(SIG_SETMASK, &sset, 0);
-}
-#endif
-
 /* The exception handling function looks like this: */
 EXCEPTION_DISPOSITION handle_exception(EXCEPTION_RECORD *,
                                        struct lisp_exception_frame *,
@@ -284,7 +276,7 @@ static void set_seh_frame(void *frame)
 
 void alloc_gc_page()
 {
-    AVER(VirtualAlloc(GC_SAFEPOINT_PAGE_ADDR, sizeof(lispobj),
+    AVER(VirtualAlloc(GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
                       MEM_RESERVE|MEM_COMMIT, PAGE_READWRITE));
 }
 
@@ -312,14 +304,14 @@ void alloc_gc_page()
 void map_gc_page()
 {
     DWORD oldProt;
-    AVER(VirtualProtect((void*) GC_SAFEPOINT_PAGE_ADDR, sizeof(lispobj),
+    AVER(VirtualProtect((void*) GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
                         PAGE_READWRITE, &oldProt));
 }
 
 void unmap_gc_page()
 {
     DWORD oldProt;
-    AVER(VirtualProtect((void*) GC_SAFEPOINT_PAGE_ADDR, sizeof(lispobj),
+    AVER(VirtualProtect((void*) GC_SAFEPOINT_PAGE_ADDR, BACKEND_PAGE_BYTES,
                         PAGE_NOACCESS, &oldProt));
 }
 
@@ -827,7 +819,7 @@ static inline boolean local_thread_stack_address_p(os_vm_address_t address)
  */
 
 os_vm_address_t
-os_validate(os_vm_address_t addr, os_vm_size_t len)
+os_validate(int movable, os_vm_address_t addr, os_vm_size_t len)
 {
     MEMORY_BASIC_INFORMATION mem_info;
 
@@ -841,20 +833,20 @@ os_validate(os_vm_address_t addr, os_vm_size_t len)
         return 0;
 
     if ((mem_info.State == MEM_RESERVE) && (mem_info.RegionSize >=len)) {
-      /* It would be correct to return here. However, support for Wine
-       * is beneficial, and Wine has a strange behavior in this
-       * department. It reports all memory below KERNEL32.DLL as
-       * reserved, but disallows MEM_COMMIT.
-       *
-       * Let's work around it: reserve the region we need for a second
-       * time. The second reservation is documented to fail on normal NT
-       * family, but it will succeed on Wine if this region is
-       * actually free.
-       */
-      VirtualAlloc(addr, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
-      /* If it is wine, the second call has succeded, and now the region
-       * is really reserved. */
-      return addr;
+        /* It would be correct to return here. However, support for Wine
+         * is beneficial, and Wine has a strange behavior in this
+         * department. It reports all memory below KERNEL32.DLL as
+         * reserved, but disallows MEM_COMMIT.
+         *
+         * Let's work around it: reserve the region we need for a second
+         * time. The second reservation is documented to fail on normal NT
+         * family, but it will succeed on Wine if this region is
+         * actually free.
+         */
+        VirtualAlloc(addr, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
+        /* If it is wine, the second call has succeded, and now the region
+         * is really reserved. */
+        return addr;
     }
 
     if (mem_info.State == MEM_RESERVE) {
@@ -864,11 +856,12 @@ os_validate(os_vm_address_t addr, os_vm_size_t len)
          * provision for MEM_RESERVE in the following code, I suppose: */
     }
 
-    if (!AVERLAX(VirtualAlloc(addr, len, (mem_info.State == MEM_RESERVE)?
-                              MEM_COMMIT: MEM_RESERVE, PAGE_EXECUTE_READWRITE)))
-        return 0;
+    os_vm_address_t actual;
 
-    return addr;
+    if (!AVERLAX(actual = VirtualAlloc(addr, len, (mem_info.State == MEM_RESERVE)?
+                                       MEM_COMMIT: MEM_RESERVE, PAGE_EXECUTE_READWRITE)))
+        return 0;
+    return actual;
 }
 
 /*
@@ -995,7 +988,7 @@ os_protect(os_vm_address_t address, os_vm_size_t length, os_vm_prot_t prot)
  * description of a space, we could probably punt this and just do
  * (FOO_START <= x && x < FOO_END) everywhere it's called. */
 static boolean
-in_range_p(os_vm_address_t a, lispobj sbeg, size_t slen)
+in_range_p(lispobj a, lispobj sbeg, size_t slen)
 {
     char* beg = (char*)((uword_t)sbeg);
     char* end = (char*)((uword_t)sbeg) + slen;
@@ -1006,18 +999,16 @@ in_range_p(os_vm_address_t a, lispobj sbeg, size_t slen)
 boolean
 is_linkage_table_addr(os_vm_address_t addr)
 {
-    return in_range_p(addr, LINKAGE_TABLE_SPACE_START, LINKAGE_TABLE_SPACE_SIZE);
+    return in_range_p((lispobj)addr, LINKAGE_TABLE_SPACE_START, LINKAGE_TABLE_SPACE_SIZE);
 }
 
 static boolean is_some_thread_local_addr(os_vm_address_t addr);
 
 boolean
-is_valid_lisp_addr(os_vm_address_t addr)
+gc_managed_addr_p(lispobj addr)
 {
-    if(in_range_p(addr, READ_ONLY_SPACE_START, READ_ONLY_SPACE_SIZE) ||
-       in_range_p(addr, STATIC_SPACE_START   , STATIC_SPACE_SIZE) ||
-       in_range_p(addr, DYNAMIC_SPACE_START  , dynamic_space_size) ||
-       is_some_thread_local_addr(addr))
+    if(gc_managed_heap_space_p(addr) ||
+       is_some_thread_local_addr((os_vm_address_t)addr))
         return 1;
     return 0;
 }
@@ -1206,7 +1197,7 @@ handle_access_violation(os_context_t *ctx,
 
     /* Safepoint pages */
 #ifdef LISP_FEATURE_SB_THREAD
-    if (fault_address == (void *) GC_SAFEPOINT_PAGE_ADDR) {
+    if (fault_address == (void *) GC_SAFEPOINT_TRAP_ADDR) {
         thread_in_lisp_raised(ctx);
         return 0;
     }
@@ -1232,6 +1223,13 @@ handle_access_violation(os_context_t *ctx,
                               MEM_COMMIT, PAGE_EXECUTE_READWRITE));
         }
         return 0;
+    } else {
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+        extern int immobile_space_handle_wp_violation(void*);
+        if (immobile_space_handle_wp_violation(fault_address)) {
+            return 0;
+        }
+#endif
     }
 
     if (fault_address == undefined_alien_address)
@@ -1239,7 +1237,7 @@ handle_access_violation(os_context_t *ctx,
 
     /* linkage table or a "valid_lisp_addr" outside of dynamic space (?) */
     if (is_linkage_table_addr(fault_address)
-        || is_valid_lisp_addr(fault_address))
+        || gc_managed_addr_p((lispobj)fault_address))
         goto try_recommit;
 
     return -1;
@@ -1856,9 +1854,7 @@ win32_maybe_interrupt_io(void* thread)
                 goto unlock;
             }
             if (ptr_CancelSynchronousIo) {
-                pthread_mutex_lock(&th->os_thread->fiber_lock);
-                done = !!ptr_CancelSynchronousIo(th->os_thread->fiber_group->handle);
-                pthread_mutex_unlock(&th->os_thread->fiber_lock);
+                done = !!ptr_CancelSynchronousIo(th->os_thread->handle);
             }
             done |= !!ptr_CancelIoEx(h,NULL);
         }
