@@ -11,8 +11,6 @@
 
 (in-package :cl-user)
 
-(load "compiler-test-util.lisp")
-
 ;;; Array initialization has complicated defaulting for :ELEMENT-TYPE,
 ;;; and both compile-time and run-time logic takes a whack at it.
 (with-test (:name (make-array :element-type :bug-126))
@@ -84,14 +82,11 @@
         do (setf (nth i list) 0)
         do (setf vector (make-array 64 :element-type 'bit
                                        :initial-contents list))
-        do (assert (= (funcall
-                       (compile nil
-                                `(lambda (rmdr)
-                                  (declare (type (simple-array bit (*)) rmdr)
-                                           (optimize (speed 3) (safety 0)))
-                                  (aref rmdr ,i)))
-                       vector)
-                      0))))
+        do (checked-compile-and-assert (:optimize '(:speed 3 :safety 0))
+               `(lambda (rmdr)
+                  (declare (type (simple-array bit (*)) rmdr))
+                  (aref rmdr ,i))
+             ((vector) 0))))
 
 ;;; Following refactoring of sequence functions to detect bad type
 ;;; specifiers, REVERSE was left broken on vectors with fill pointers.
@@ -214,15 +209,18 @@
 (let ((array (make-array nil :initial-contents nil)))
   (assert (eql (aref array) nil)))
 
-(let ((f (compile nil '(lambda ()
-                        (let ((a (make-array '(4)
-                                             :element-type 'base-char
-                                             :initial-element #\z)))
-                          (setf (aref a 0) #\a)
-                          (setf (aref a 1) #\b)
-                          (setf (aref a 2) #\c)
-                          a)))))
-  (assert (= (length (funcall f)) 4)))
+(with-test (:name (make-array (setf aref) length))
+  (checked-compile-and-assert ()
+      '(lambda ()
+         (let ((a (make-array '(4)
+                              :element-type 'base-char
+                              :initial-element #\z)))
+           (setf (aref a 0) #\a)
+           (setf (aref a 1) #\b)
+           (setf (aref a 2) #\c)
+           a))
+    (() 4 :test (lambda (values expected)
+                  (= (length (first values)) (first expected))))))
 
 (let ((x (make-array nil :initial-element 'foo)))
   (adjust-array x nil)
@@ -231,19 +229,20 @@
 ;;; BUG 315: "no bounds check for access to displaced array"
 ;;;  reported by Bruno Haible sbcl-devel "various SBCL bugs" from CLISP
 ;;;  test suite.
-(locally (declare (optimize (safety 3) (speed 0)))
-  (let* ((x (make-array 10 :fill-pointer 4 :element-type 'character
-                        :initial-element #\space :adjustable t))
-         (y (make-array 10 :fill-pointer 4 :element-type 'character
-                        :displaced-to x)))
-    (assert (eq x (adjust-array x '(5))))
-    (assert (eq :error (handler-case
-                           (char y 0)
-                         (sb-int:invalid-array-error (e)
-                           (assert (eq y (type-error-datum e)))
-                           (assert (equal `(vector character 10)
-                                          (type-error-expected-type e)))
-                           :error))))))
+(with-test (:name (:displaced-to aref sb-int:invalid-array-index-error :bug-315))
+  (locally (declare (optimize (safety 3) (speed 0)))
+    (let* ((x (make-array 10 :fill-pointer 4 :element-type 'character
+                          :initial-element #\space :adjustable t))
+           (y (make-array 10 :fill-pointer 4 :element-type 'character
+                          :displaced-to x)))
+      (assert (eq x (adjust-array x '(5))))
+      (assert (eq :error (handler-case
+                             (char y 0)
+                           (sb-int:invalid-array-error (e)
+                             (assert (eq y (type-error-datum e)))
+                             (assert (equal `(vector character 10)
+                                            (type-error-expected-type e)))
+                             :error)))))))
 
 ;;; MISC.527: bit-vector bitwise operations used LENGTH to get a size
 ;;; of a vector
@@ -363,15 +362,9 @@
                     #(5 10)))))
 
 (with-test (:name (:make-array-transform-unknown-type :bug-1156095))
-  (assert
-   (handler-case
-       (compile nil `(lambda () (make-array '(1 2)
-                                            :element-type ',(gensym))))
-     (style-warning ()
-       t)
-     (:no-error (&rest args)
-       (declare (ignore args))
-       nil))))
+  (assert (nth-value 3 (checked-compile
+                        `(lambda () (make-array '(1 2) :element-type ',(gensym)))
+                        :allow-style-warnings t))))
 
 (with-test (:name :dont-make-array-bad-keywords)
   ;; This used to get a heap exhaustion error because of trying
@@ -386,26 +379,42 @@
        (string= (simple-condition-format-control c)
                 "Can't specify both :INITIAL-ELEMENT and :INITIAL-CONTENTS")))))
 
-(with-test (:name :make-array-sanity-check-dims-first)
+(with-test (:name (make-array :sanity-check-dims-first))
   ;; A full call to %MAKE-ARRAY will signal a TYPE-ERROR on these inputs
   ;; instead of trying to consume a massive amount of memory.
   ;; Additionally, the relevent IR1 transform should give up.
-  (locally
-    (declare (notinline make-array))
-    (assert-error (make-array `(-1 -1 ,(- (ash array-dimension-limit -2) 4)))
-                  type-error))
-  (locally
-    (declare (inline make-array))
-    (assert-error (make-array `(-1 -1 ,(- (ash array-dimension-limit -2) 4)))
-                  type-error)))
+  (flet ((test (inline)
+           (multiple-value-bind (fun failure-p warnings)
+               (checked-compile
+                `(lambda ()
+                   (declare (,inline make-array))
+                   (make-array `(-1 -1 ,(- (ash array-dimension-limit -2) 4))))
+                :allow-failure t :allow-warnings t)
+             (ecase inline
+               (inline
+                (assert failure-p)
+                (assert (= 1 (length warnings))))
+               (notinline
+                (assert failure-p)
+                (assert (= 1 (length warnings)))))
+             (assert-error (funcall fun) type-error))))
+    (test 'inline)
+    (test 'notinline)))
 
-(with-test (:name :make-array-size-overflow)
+(with-test (:name (make-array :size-overflow))
   ;; 1-bit fixnum tags make array limits overflow the word length
   ;; when converted to bytes
   (when (= sb-vm:n-fixnum-tag-bits 1)
-    (assert-error (make-array (1- array-total-size-limit)) error)))
+    (multiple-value-bind (fun failure-p warnings)
+        (checked-compile
+         '(lambda ()
+            (make-array (1- array-total-size-limit)))
+         :allow-failure t :allow-warnings t)
+      (assert failure-p)
+      (assert (= 1 (length warnings)))
+      (assert-error (funcall fun) type-error))))
 
-(with-test (:name :adjust-non-adjustable-array)
+(with-test (:name (adjust-array :non-adjustable))
   (let* ((a (make-array '(2 3) :initial-contents '((0 1 2) (3 4 5))))
          (b (adjust-array a '(2 2))))
     (setf (aref a 0 0) 11)
@@ -413,40 +422,34 @@
     (assert (not (eq a b)))))
 
 (with-test (:name :check-bound-elision)
-  (assert-error (funcall (checked-compile
-                          `(lambda (x)
-                             (char "abcd" x)))
-                         4)
-                sb-int:invalid-array-index-error)
-  (assert (eql (funcall (checked-compile
-                         `(lambda (x)
-                            (declare (optimize (safety 0)))
-                            ;; Strings are null-terminated for C interoperability
-                            (char "abcd" x)))
-                        4)
-               #\Nul)))
+  (checked-compile-and-assert (:optimize :safe)
+      `(lambda (x)
+         (char "abcd" x))
+    ((4) (condition 'sb-int:invalid-array-index-error)))
+  (checked-compile-and-assert (:optimize '(:safety 0))
+      `(lambda (x)
+         ;; Strings are null-terminated for C interoperability
+         (char "abcd" x))
+    ((4) #\Nul)))
 
-(with-test (:name :adjust-array-transform)
-  (assert (equalp (funcall
-                  (checked-compile
-                   `(lambda ()
-                      (adjust-array #(1 2 3) 3 :displaced-to #(4 5 6)))))
-                 #(4 5 6))))
+(with-test (:name (adjust-array :transform))
+  (checked-compile-and-assert ()
+      `(lambda ()
+         (adjust-array #(1 2 3) 3 :displaced-to #(4 5 6)))
+    (() #(4 5 6) :test #'equalp)))
 
-(with-test (:name :adjust-array-fill-pointer)
+(with-test (:name (adjust-array :fill-pointer))
   (let ((array (make-array 10 :fill-pointer t)))
     (assert (= (fill-pointer (adjust-array array 5 :fill-pointer 2))
                2))))
 
-(with-test (:name :adjust-array-initial-element)
-  (assert (equal (funcall
-                  (checked-compile
-                   `(lambda (x)
-                      (adjust-array x 5 :initial-element #\x)))
-                  "abc")
-                 "abcxx")))
+(with-test (:name (adjust-array :initial-element))
+  (checked-compile-and-assert ()
+      `(lambda (x)
+         (adjust-array x 5 :initial-element #\x))
+    (("abc") "abcxx")))
 
-(with-test (:name :array-initial-contents-1)
+(with-test (:name (make-array :initial-contents 1))
   (flet ((f (x y)
            (sb-int:dx-let ((a (make-array `(,x ,y)
                                           :initial-contents
@@ -456,7 +459,7 @@
     (f 2 3)
     (assert-error (f 3 2))))
 
-(with-test (:name :array-initial-contents-2)
+(with-test (:name (make-array :initial-contents 2))
   (labels ((compute-contents () '((a b c) (1 2 3)))
            (f (x y)
              (sb-int:dx-let ((a (make-array `(,x ,y)
@@ -468,43 +471,42 @@
     (f 2 3)
     (assert-error (f 3 2))))
 
-(with-test (:name :array-initial-contents-3)
-  (multiple-value-bind (f warningp errorp)
-      ;; FIXME: should be CHECKED-COMPILE
-      (let ((*error-output* (make-broadcast-stream)))
-        (compile nil '(lambda (z)
-                        (symbol-macrolet ((x (+ 1 1)) (y (* 2 1)))
-                          (make-array `(,x ,y)
-                                      :initial-contents
-                                      `((,z ,z 1) (,z ,z ,z)))))))
-    (assert (and f warningp errorp))))
+(with-test (:name (make-array :initial-contents 3))
+  (multiple-value-bind (fun failure-p warnings)
+      (checked-compile
+       '(lambda (z)
+          (symbol-macrolet ((x (+ 1 1)) (y (* 2 1)))
+            (make-array `(,x ,y)
+                        :initial-contents
+                        `((,z ,z 1) (,z ,z ,z)))))
+       :allow-failure t :allow-warnings t)
+    (assert failure-p)
+    (assert (= 1 (length warnings)))
+    (assert-error (funcall fun) error)))
 
-(with-test (:name :adjust-array-element-type)
-  (let ((fun (checked-compile '(lambda (array)
-                                (adjust-array array 3 :element-type '(signed-byte 2))))))
-    (assert-error (funcall fun #(1 2 3))))
-  (let ((fun (checked-compile '(lambda (array)
-                                (adjust-array array 5 :displaced-to #(1 2 3))))))
-    (assert-error (funcall fun (make-array 5 :adjustable t :element-type 'fixnum)))))
+(with-test (:name (adjust-array :element-type))
+  (checked-compile-and-assert ()
+      `(lambda (array)
+         (adjust-array array 3 :element-type '(signed-byte 2)))
+    ((#(1 2 3)) (condition 'error)))
+  (checked-compile-and-assert ()
+      `(lambda (array)
+         (adjust-array array 5 :displaced-to #(1 2 3)))
+    (((make-array 5 :adjustable t :element-type 'fixnum)) (condition 'error))))
 
-(with-test (:name :make-array-transform-fill-pointer-nil)
-  (let ((fun (checked-compile '(lambda ()
-                                (make-array 3 :fill-pointer nil)))))
-    (assert (not (ctu:find-named-callees fun :name 'sb-kernel:%make-array))))
-  (let ((fun (checked-compile '(lambda ()
-                                (make-array 3 :fill-pointer t)))))
-    (assert (not (ctu:find-named-callees fun :name 'sb-kernel:%make-array))))
-  (let ((fun (checked-compile '(lambda ()
-                                (make-array 3 :adjustable nil)))))
-    (assert (not (ctu:find-named-callees fun :name 'sb-kernel:%make-array))))
-  (let ((fun (checked-compile '(lambda ()
-                                (make-array '(3 3) :adjustable nil)))))
-    (assert (not (ctu:find-named-callees fun :name 'sb-kernel:%make-array))))
-  (let ((fun (checked-compile '(lambda ()
-                                (make-array '(3 3) :fill-pointer nil)))))
-    (assert (not (ctu:find-named-callees fun :name 'sb-kernel:%make-array)))))
+(with-test (:name (make-array :transform :fill-pointer nil))
+  (flet ((test (form)
+           (let ((fun (checked-compile `(lambda () ,form))))
+             (assert (not (ctu:find-named-callees
+                           fun :name 'sb-kernel:%make-array))))))
+    (test '(make-array 3      :fill-pointer nil))
+    (test '(make-array 3      :fill-pointer nil))
+    (test '(make-array 3      :fill-pointer t))
+    (test '(make-array 3      :adjustable nil))
+    (test '(make-array '(3 3) :adjustable nil))
+    (test '(make-array '(3 3) :fill-pointer nil))))
 
-(with-test (:name (:make-array-transform :adjustable :fill-pointer))
+(with-test (:name (make-array :transform :adjustable :fill-pointer))
   (let ((fun (checked-compile '(lambda (fp)
                                 (make-array 3 :adjustable t :fill-pointer fp)))))
     (assert (not (ctu:find-named-callees fun :name 'sb-kernel:%make-array)))
@@ -516,3 +518,8 @@
     (assert-error (funcall fun 'abc))
     (assert (not (array-has-fill-pointer-p (funcall fun nil))))
     (assert (= (length (funcall fun nil)) 3))))
+
+(with-test (:name :check-bound-fixnum-check)
+  (checked-compile-and-assert (:optimize :safe)
+      `(lambda (x) (aref #100(a) x))
+    ((#\Nul) (condition 'type-error))))
