@@ -162,6 +162,15 @@
                   (make-array (do)))
                :allow-failure t))))
 
+(with-test (:name (make-array :dimensions :bad-propagated-value))
+  (assert
+   (nth-value 1
+              (checked-compile
+               `(lambda ()
+                  (let ((x '(("foo"))))
+                    (make-array (list x) :fill-pointer 0)))
+               :allow-warnings t))))
+
 (with-test (:name :&rest-ref-bad-n)
   (assert
    (nth-value 1
@@ -300,4 +309,19 @@
                          (svref x *break-on-signals*))
                       :allow-warnings t))))
 
+(with-test (:name :unknown-keys-propagation-error-checking.1)
+  (assert (nth-value 1
+                     (checked-compile
+                      `(lambda (x)
+                         (let ((a :tests))
+                           (find 1 x a #'eql)))
+                      :allow-warnings t))))
+
+(with-test (:name :unknown-keys-propagation-error-checking.2)
+  (assert (nth-value 1
+                     (checked-compile
+                      `(lambda ()
+                         (apply 'find '(3 (1 2 3) :bad t)))
+                      :allow-warnings t))))
+
 
