@@ -1306,3 +1306,76 @@
      '(lambda ()
        (make-array 2 :initial-element 10))
      (() #(10 10) :test #'equalp))))
+
+(with-test (:name :deleted-tail-sets)
+  (checked-compile-and-assert
+   ()
+   '(lambda ()
+     (labels ((f (&optional (a (catch t 6))
+                            (b (error ""))
+                            (c (unwind-protect 1)))
+                (+ a b c)))
+       (unwind-protect (f 4))))
+   (() (condition 'error))))
+
+;;; The SLEEP source transform barfed on float positive infinity
+;;; values.
+(with-test (:name (compile sleep float :infinity :lp-1754081))
+  (checked-compile '(lambda () (sleep single-float-positive-infinity)))
+  (checked-compile '(lambda () (sleep double-float-positive-infinity))))
+
+(with-test (:name :atanh-type-derivation)
+  (checked-compile-and-assert
+   ()
+   '(lambda (x)
+     (atanh (coerce x '(double-float * (0.0d0)))))))
+
+(with-test (:name :ir1-optimize-combination-unknown-keys)
+  (checked-compile-and-assert
+      ()
+      '(lambda (p x y)
+        (let ((f (when p #'string-equal)))
+          (when f
+            (funcall f "a" "b" x y))))
+    ((t :start1 0) nil)))
+
+(with-test (:name :member-transform)
+  (let ((list '(2 1 3)))
+    (checked-compile-and-assert
+     ()
+     '(lambda (list &key key)
+       (member 1 list :key key))
+     ((list) (cdr list)))))
+
+(with-test (:name :note-no-stack-allocation-casts)
+  (checked-compile-and-assert
+   ()
+   `(lambda ()
+     (let ((*s* (the integer (catch 'ct1 0))))
+       (declare (dynamic-extent *s*)
+                (special *s*))))))
+
+(with-test (:name :dxify-downward-funargs-variable-name)
+  (checked-compile-and-assert
+      ()
+      '(lambda () ((lambda (map) (funcall map)) #'list))))
+
+(with-test (:name :dxify-downward-funargs-malformed)
+  (checked-compile
+      '(lambda () (sb-debug::map-backtrace))
+      :allow-style-warnings t))
+
+(with-test (:name :dxify-downward-funargs-casts)
+  (checked-compile-and-assert
+   ()
+   '(lambda (f x)
+     (flet ((f (y) (funcall f y)))
+       (funcall (the (satisfies eval) #'every) #'f x)))
+   ((#'evenp '(2 2 4)) t)))
+
+(with-test (:name :array-call-type-deriver-non-fun-type)
+  (checked-compile-and-assert
+      ()
+      '(lambda (x) (funcall (the compiled-function #'aref) x))
+    ((#0A123) 123)))
+
