@@ -17,11 +17,9 @@
 
 (/show "beginning tests/info.before-xc.lisp")
 
-(assert (eq (sb!int:info :variable :kind 'sb!vm:vector-data-offset)
-            :constant))
 ;;; It's possible in general for a constant to have the value NIL, but
 ;;; not for vector-data-offset, which must be a number:
-(assert (boundp 'sb!vm:vector-data-offset))
+(assert (constantp 'sb!vm:vector-data-offset))
 (assert (integerp (symbol-value 'sb!vm:vector-data-offset)))
 
 (in-package "SB!IMPL")
@@ -128,8 +126,4 @@
               ((CAS ,s) (3 . "CAS-info#3"))
               ((FROB ,s) (15 . "FROB-info#15"))))))
 
-(let ((ht (make-info-hashtable)))
-  (setf (info-gethash '(hairy name) ht) :whatever)
-  (assert (eq (info-gethash (list 'hairy 'name) ht) :whatever)))
-
 (/show "done with tests/info.before-xc.lisp")
