@@ -11,15 +11,10 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-(in-package :cl-user)
-
-(load "test-util.lisp")
-(load "compiler-test-util.lisp")
-(use-package :test-util)
 
 ;;;; properties of symbols, e.g. presence of doc strings for public symbols
 
-(with-test (:name (documentation :cl) :skipped-on '(:not :sb-doc))
+(with-test (:name (documentation :cl) :skipped-on (:not :sb-doc))
   (let ((n 0))
     (do-symbols (s 'cl)
       (if (fboundp s)
@@ -45,16 +40,6 @@
   (assert (< 0
              (length (apropos-list "" "SB-VM" t))
              (length (apropos-list "" "SB-VM")))))
-
-;;; DESCRIBE shouldn't fail on rank-0 arrays (bug reported and fixed
-;;; by Lutz Euler sbcl-devel 2002-12-03)
-(with-test (:name (describe array :rank 0))
-  (flet ((test (array)
-           (assert (plusp (length (with-output-to-string (stream)
-                                    (describe array stream)))))))
-    (test #0a0)
-    (test #(1 2 3))
-    (test #2a((1 2) (3 4)))))
 
 ;;; TYPEP, SUBTYPEP, UPGRADED-ARRAY-ELEMENT-TYPE and
 ;;; UPGRADED-COMPLEX-PART-TYPE should be able to deal with NIL as an
@@ -89,7 +74,7 @@
 
 ;;; SLEEP should not cons except on 32-bit platforms when
 ;;; (> (mod seconds 1) (* most-positive-fixnum 1e-9))
-(with-test (:name (sleep :non-consing) :fails-on :win32
+(with-test (:name (sleep :non-consing)
                   :skipped-on :interpreter)
   (handler-case (sb-ext:with-timeout 5
                   (ctu:assert-no-consing (sleep 0.00001s0))
@@ -168,7 +153,7 @@
 ;;; comprehensive test.
 (with-test (:name (sb-ext:gc :minimal :stress))
   (loop repeat 2
-     do (compile nil '(lambda (x) x))
+     do (checked-compile '(lambda (x) x))
      do (sb-ext:gc :full t)))
 
 ;;; On x86-64, the instruction definitions for CMP*[PS][SD] were broken
@@ -221,22 +206,13 @@
     (assert (string= string1 string2)))))
 
 (with-test (:name :disassemble-assembly-routine)
-  (let ((code
-         (block nil
-           (sb-vm::map-allocated-objects
-            (lambda (obj type size)
-              (declare (ignore size))
-              (when (= type sb-vm:code-header-widetag)
-                (return obj)))
-            :read-only))))
-    (assert code) ; found something to disassemble
-    (sb-disassem:disassemble-code-component code
-     :stream (make-broadcast-stream))))
+  (sb-disassem:disassemble-code-component sb-fasl:*assembler-routines*
+     :stream (make-broadcast-stream)))
 
 ;;; This tests that the x86-64 disasembler does not crash
 ;;; on LEA with a rip-relative operand and no label.
-(with-test (:name :disassemble-no-labels
-                  :skipped-on '(not :x86-64))
+(with-test (:name (disassemble :no-labels)
+                  :skipped-on (not :x86-64))
   (let* ((lines
           (split-string
            (with-output-to-string (stream)
@@ -277,14 +253,12 @@
 (with-test (:name :bug-1095483)
   (assert-error (fboundp '(cas "foo"))))
 
-(with-test (:name :sleep-return-value)
-  (assert (equal (multiple-value-list
-                  (funcall
-                   (checked-compile `(lambda ()
-                                       (sleep 0.1)))))
-                 '(nil))))
+(with-test (:name (sleep :return-value))
+  (checked-compile-and-assert ()
+      `(lambda () (sleep 0.001))
+    (() nil)))
 
-(with-test (:name :time-no-print-length-abbreviation)
+(with-test (:name (time :no *print-length* :abbreviation))
   (let ((s (make-string-output-stream)))
     (let ((*trace-output* s))
       (time (progn)))
@@ -295,3 +269,22 @@
 (with-test (:name :split-seconds-for-sleep)
   (assert (< (nth-value 1 (sb-impl::split-seconds-for-sleep 7.2993028420866d7))
              1000000000)))
+#+x86-64
+(with-test (:name :restart-invalid-arg-counts.1)
+  (handler-bind ((error (lambda (c)
+                          (invoke-restart (find-restart 'sb-kernel::replace-function c) 'list))))
+    (assert (equal (eval '(cons 324)) '(324)))))
+
+#+x86-64
+(with-test (:name :restart-invalid-arg-counts.2)
+  (handler-bind ((error (lambda (c)
+                          (invoke-restart (find-restart 'sb-kernel::call-form c) 123))))
+    (assert (= (eval '(cons 1)) 123))))
+
+(with-test (:name :restart-bogus-arg-to-values-list-error)
+  (let ((fun (checked-compile `(lambda (x) (values-list x)))))
+    (assert (equal (handler-bind ((sb-kernel::values-list-argument-error
+                                   #'continue))
+                     (multiple-value-list
+                      (funcall fun '(1 2 3 4 5 6 7 8 . 10))))
+                   '(1 2 3 4 5 6 7 8)))))
